import csv
import json
import logging
import os
import pkgutil
import re
import sys
import traceback
import xml.etree.ElementTree as ET
import zipfile
from importlib import import_module
from importlib.machinery import SourceFileLoader
from io import StringIO

from . import repo

logger = logging.getLogger("faraday").getChild(__name__)


class ReportAnalyzer:

    def __init__(self, plugin_manager):
        self.plugin_manager = plugin_manager

    def get_plugin(self, report_path):
        plugin = None
        if not os.path.isfile(report_path):
            logger.error("Report [%s] don't exists", report_path)
            return plugin
        else:
            file_name = os.path.basename(report_path)
            plugin = self._get_plugin_by_name(file_name)
            if not plugin:  # Was unable to detect plugin from report file name
                logger.debug("Plugin by name not found")
                plugin = self._get_plugin_by_file_type(report_path)
                if not plugin:
                    logger.debug("Plugin by file not found")
        if not plugin:
            logger.debug("Plugin for file (%s) not found", report_path)
        return plugin

    def _get_plugin_by_name(self, file_name_base):
        plugin_id = None
        plugin_name_regex = r".*_faraday_(?P<plugin_name>.+)\..*$"
        match = re.match(plugin_name_regex, file_name_base)
        if match:
            plugin_id = match.groupdict()['plugin_name'].lower()
            logger.debug("Plugin name match: %s", plugin_id)
            plugin = self.plugin_manager.get_plugin(plugin_id)
            if plugin:
                logger.debug("Plugin by name Found: %s", plugin.id)
                return plugin
            else:
                logger.debug("Invalid plugin from file name: %s", plugin_id)
                return None
        else:
            logger.debug("Could not extract plugin_id from filename: %s", file_name_base)
            return plugin_id

    def _get_plugin_by_file_type(self, report_path):
        plugin = None
        file_name = os.path.basename(report_path)
        file_name_base, file_extension = os.path.splitext(file_name)
        file_extension = file_extension.lower()
        main_tag = None
        main_tag_attributes = {}
        file_csv_headers = set()
        file_json_keys = set()
        files_in_zip = set()
        logger.debug("Analyze report File")
        # Try to parse as xml
        try:
            report_file = open(report_path, "rb")
        except Exception as e:
            logger.error("Error reading report content [%s]", e)
        else:
            try:
                for event, elem in ET.iterparse(report_file, ('start',)):
                    prefix, has_namespace, postfix = elem.tag.partition("}")
                    if has_namespace:
                        main_tag = postfix
                    else:
                        main_tag = elem.tag
                        try:
                            main_tag_attributes = elem.attrib
                        except AttributeError:
                            pass
                    break
                logger.debug("Found XML content on file: %s - Main tag: %s Attributes: %s", report_path, main_tag,
                             main_tag_attributes)
            except Exception as e:
                logger.debug("Non XML content [%s] - %s", report_path, e)
                try:
                    report_file.seek(0)
                    json_data = json.load(report_file)
                    if isinstance(json_data, list):
                        if len(json_data) > 0:
                            file_json_keys = set(json_data[0].keys())
                    else:
                        file_json_keys = set(json_data.keys())
                    logger.debug("Found JSON content on file: %s - Keys: %s", report_path, file_json_keys)
                except Exception as e:
                    logger.debug("Non JSON content [%s] - %s", report_path, e)
                    try:
                        report_file.seek(0)
                        reader_file_string = StringIO(report_file.read().decode('utf-8'))
                        reader = csv.DictReader(reader_file_string)
                        file_csv_headers = set(reader.fieldnames)
                        logger.debug("Found CSV content on file: %s - Keys: %s", report_path, file_json_keys)
                    except Exception as e:
                        logger.debug("Non CSV content [%s] - %s", report_path, e)
                        try:
                            file_zip = zipfile.ZipFile(report_path, "r")
                            files_in_zip = set(file_zip.namelist())
                            logger.debug("List of files found in ZIP %s", file_zip)
                        except Exception as e:
                            logger.debug("Non ZIP content [%s] - %s", report_path, e)
            finally:
                report_file.close()
                for _plugin_id, _plugin in self.plugin_manager.get_plugins():
                    logger.debug("Try plugin: %s", _plugin_id)
                    try:
                        if _plugin.report_belongs_to(main_tag=main_tag, main_tag_attributes=main_tag_attributes,
                                                     report_path=report_path, extension=file_extension,
                                                     file_json_keys=file_json_keys, file_csv_headers=file_csv_headers,
                                                     files_in_zip=files_in_zip):
                            plugin = _plugin
                            logger.debug("Plugin by File Found: %s", plugin.id)
                            break
                    except Exception as e:
                        logger.error("Error in plugin analysis: (%s) %s", _plugin_id, e)
        return plugin


class CommandAnalyzer:

    def __init__(self, plugin_manager):
        self.plugin_manager = plugin_manager

    def get_plugin(self, command_string):
        plugin = None
        logger.debug("Look plugin for command: %s", command_string)
        for _plugin_id, _plugin in self.plugin_manager.get_plugins():
            logger.debug("Try plugin: %s", _plugin_id)
            try:
                if _plugin.canParseCommandString(command_string):
                    plugin = _plugin
            except Exception as e:
                logger.error("Error in plugin analysis: (%s) %s", _plugin_id, e)
        return plugin


class PluginsManager:

    def __init__(self, custom_plugins_folder=None, **kwargs):
        self.kwargs = kwargs
        self.plugins = {}
        self.plugin_modules = {}
        self._load_plugins(custom_plugins_folder)

    def _load_plugins(self, custom_plugins_folder):
        logger.info("Loading Native Plugins...")
        if not self.plugins:
            for _, name, _ in filter(lambda x: x[2], pkgutil.iter_modules(repo.__path__)):
                try:
                    plugin_module = import_module(f"faraday_plugins.plugins.repo.{name}.plugin")
                    if hasattr(plugin_module, "createPlugin"):
                        plugin_instance = plugin_module.createPlugin()
                        plugin_id = plugin_instance.id.lower()
                        if not plugin_instance.auto_load:
                            logger.debug("Skip load plugin [%s]", plugin_id)
                            continue
                        if plugin_id not in self.plugin_modules:
                            self.plugin_modules[plugin_id] = plugin_module
                            logger.debug("Load Plugin [%s]", name)
                        else:
                            logger.debug("Plugin already loaded [%s]", plugin_id)
                    else:
                        logger.error("Invalid Plugin [%s]", name)
                except Exception as e:
                    logger.error("Cant load plugin module: %s [%s]", name, e)
            if custom_plugins_folder:
                if os.path.isdir(custom_plugins_folder):
                    logger.info("Loading Custom Plugins...")
                    dir_name_regexp = re.compile(r"^[\d\w\-\_]+$")
                    for name in os.listdir(custom_plugins_folder):
                        if dir_name_regexp.match(name) and name != "__pycache__":
                            module_path = os.path.join(custom_plugins_folder, name)
                            module_filename = os.path.join(module_path, "plugin.py")
                            try:
                                sys.path.append(module_path)
                                file_ext = os.path.splitext(module_filename)[1]
                                if file_ext.lower() == '.py':
                                    if name not in self.plugin_modules:
                                        loader = SourceFileLoader(name, module_filename)
                                        plugin_module = loader.load_module()
                                        plugin_instance = plugin_module.createPlugin()
                                        plugin_id = plugin_instance.id.lower()
                                        if not plugin_instance.auto_load:
                                            logger.info("Skip load plugin [%s]", plugin_id)
                                            continue
                                        if plugin_id not in self.plugin_modules:
                                            self.plugin_modules[plugin_id] = plugin_module
                                    else:
                                        logger.debug("Plugin with same name already loaded [%s]", name)
                                logger.debug("Load Plugin [%s]", name)
                            except Exception as e:
                                logger.debug("An error ocurred while loading plugin %s.\n%s", module_filename,
                                             traceback.format_exc())
                                logger.warning(e)
                else:
                    logger.warning("Invalid custom plugins folder [%s]", custom_plugins_folder)
            logger.info("%s plugins loaded", len(self.plugin_modules))

    def get_plugin(self, plugin_id):
        plugin = None
        plugin_id = plugin_id.lower()
        if plugin_id in self.plugin_modules:
            plugin = self.plugin_modules[plugin_id].createPlugin(**self.kwargs)
        else:
            logger.debug("Unknown Plugin: %s", plugin_id)
        return plugin

    def get_plugins(self):
        for plugin_id, plugin_module in self.plugin_modules.items():
            logger.debug("Instance Plugin: %s", plugin_id)
            yield plugin_id, plugin_module.createPlugin(**self.kwargs)



""""
Faraday Penetration Test IDE
Copyright (C) 2013  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information

"""
import sys
import re
import xml.etree.ElementTree as ET

from bs4 import BeautifulSoup

from faraday_plugins.plugins.plugin import PluginXMLFormat
from faraday_plugins.plugins.plugins_utils import CVE_regex

__author__ = "Francisco Amato"
__copyright__ = "Copyright (c) 2013, Infobyte LLC"
__credits__ = ["Francisco Amato"]
__license__ = ""
__version__ = "1.0.0"
__maintainer__ = "Francisco Amato"
__email__ = "famato@infobytesec.com"
__status__ = "Development"


class NetsparkerXmlParser:
    """
    The objective of this class is to parse an xml file generated by the netsparker tool.

    TODO: Handle errors.
    TODO: Test netsparker output version. Handle what happens if the parser doesn't support it.
    TODO: Test cases.

    @param netsparker_xml_filepath A proper xml generated by netsparker
    """

    def __init__(self, xml_output, plugin):
        self.filepath = xml_output
        self.plugin = plugin

        tree = self.parse_xml(xml_output)
        if tree:
            self.items = [data for data in self.get_items(tree)]
        else:
            self.items = []

    def parse_xml(self, xml_output):
        """
        Open and parse an xml file.

        TODO: Write custom parser to just read the nodes that we need instead of
        reading the whole file.

        @return xml_tree An xml tree instance. None if error.
        """
        try:
            tree = ET.fromstring(xml_output)
        except SyntaxError as err:
            self.plugin.logger.error(f"SyntaxError: {err}. {xml_output}")
            return None

        return tree

    def get_items(self, tree):
        """
        @return items A list of Host instances
        """
        for node in tree.findall("vulnerability"):
            yield Item(node)


class Item:
    """
    An abstract representation of a Item


    @param item_node A item_node taken from an netsparker xml tree
    """

    def __init__(self, item_node):
        self.node = item_node
        self.url = self.get_text_from_subnode("url")
        host = re.search(
            r"(http|https|ftp)\://([a-zA-Z0-9\.\-]+(\:[a-zA-Z0-9\.&amp;%\$\-]+)*@)*((25[0-5]|2[0-4][0-9]|[0-1]{1}[0-9]"
            r"{2}|[1-9]{1}[0-9]{1}|[1-9])\.(25[0-5]|2[0-4][0-9]|[0-1]{1}[0-9]{2}|[1-9]{1}[0-9]{1}|[1-9]|0)\.(25[0-5]|2"
            r"[0-4][0-9]|[0-1]{1}[0-9]{2}|[1-9]{1}[0-9]{1}|[1-9]|0)\.(25[0-5]|2[0-4][0-9]|[0-1]{1}[0-9]{2}|[1-9]{1}[0-9]"
            r"{1}|[0-9])|localhost|([a-zA-Z0-9\-]+\.)*[a-zA-Z0-9\-]+\.(com|edu|gov|int|mil|net|org|biz|arpa|info|name|"
            "pro|aero|coop|museum|[a-zA-Z]{2}))[\\:]*([0-9]+)*([/]*($|[\\(\\)a-zA-Z0-9\\.\\,\\?\'\\\\+&amp;%\\$#\\=~_\\-]+)).*?$",
            self.url)
        self.protocol = host.group(1)
        self.hostname = host.group(4)
        self.port = 80

        if self.protocol == 'https':
            self.port = 443
        if host.group(11) is not None:
            self.port = host.group(11)

        self.name = self.get_text_from_subnode("type")
        self.name_title = self.get_text_from_subnode("title")
        self.desc = self.get_text_from_subnode("description")
        self.severity = self.re_map_severity(self.get_text_from_subnode("severity"))
        self.certainty = self.get_text_from_subnode("certainty")
        self.method = self.get_text_from_subnode("vulnerableparametertype")
        self.param = self.get_text_from_subnode("vulnerableparameter")
        self.paramval = self.get_text_from_subnode("vulnerableparametervalue")
        self.reference = self.get_text_from_subnode("externalReferences")
        remedy = self.get_text_from_subnode("remedy")
        self.resolution = self.get_text_from_subnode("actionsToTake") + remedy if remedy else ""
        self.request = self.get_text_from_subnode("rawrequest")
        self.response = self.get_text_from_subnode("rawresponse")
        self.kvulns = []
        self.cve = []
        for v in self.node.findall("knownvulnerabilities/knownvulnerability"):
            self.node = v
            recheck = CVE_regex.search(v.find('title').text)
            if recheck:
                self.cve.append(recheck.group())
            self.kvulns.append(self.get_text_from_subnode(
                "severity") + "-" + self.get_text_from_subnode("title"))

        self.extra = []
        for v in item_node.findall("extrainformation/info"):
            name_tag = v.find('name')
            value_tag = v.find('value')
            if name_tag is not None:
                self.extra.append(f"{name_tag.text}:{value_tag.text}")

        self.node = item_node
        self.node = item_node.find("classification")
        self.owasp = self.get_text_from_subnode("OWASP")
        self.wasc = self.get_text_from_subnode("WASC")
        self.cwe = self.get_text_from_subnode("CWE")
        self.capec = self.get_text_from_subnode("CAPEC")
        self.pci = self.get_text_from_subnode("PCI")
        self.pci2 = self.get_text_from_subnode("PCI2")
        self.node = item_node.find("classification/CVSS")
        self.cvss_full_vector = self.get_text_from_subnode("vector")
        self.cvss_score = self.get_text_from_subnode("score[1]/value") if self.get_text_from_subnode("score[1]/value") else None
        self.cvss3 = {}
        self.ref = []
        if self.cwe:
            self.cwe = ["CWE-" + self.cwe]
        if self.owasp:
            self.ref.append("OWASP-" + self.owasp)
        if self.reference:
            self.ref.extend(sorted(set(re.findall(r'https?://(?:[-\w.]|(?:%[\da-fA-F]{2}))+', self.reference))))
        if self.cvss_full_vector:
            self.cvss3["vector_string"] = self.cvss_full_vector
        self.data = ""
        self.data += "\nKnowVulns: " + \
            "\n".join(self.kvulns) if self.kvulns else ""
        self.data += "\nWASC: " + self.wasc if self.wasc else ""
        self.data += "\nCertainty: " + self.certainty if self.certainty else ""
        self.data += "\nPCI: " + self.pci if self.pci else ""
        self.data += "\nPCI2: " + self.pci2 if self.pci2 else ""
        self.data += "\nCAPEC: " + self.capec if self.capec else ""
        self.data += "\nPARAM: " + self.param if self.param else ""
        self.data += "\nPARAM VAL: " + \
            repr(self.paramval) if self.paramval else ""
        self.data += "\nExtra: " + "\n".join(self.extra) if self.extra else ""

    @staticmethod
    def re_map_severity(severity):
        if severity == "Important":
            return "high"
        return severity

    def get_text_from_subnode(self, subnode_xpath_expr):
        """
        Finds a subnode in the host node and the retrieves a value from it.

        @return An attribute value
        """
        if self.node:
            sub_node = self.node.find(subnode_xpath_expr)
            if sub_node is not None:
                return sub_node.text
        return None


class NetsparkerPlugin(PluginXMLFormat):
    """
    Example plugin to parse netsparker output.
    """

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.identifier_tag = "netsparker"
        self.id = "Netsparker"
        self.name = "Netsparker XML Output Plugin"
        self.plugin_version = "0.0.1"
        self.version = "Netsparker 3.1.1.0"
        self.framework_version = "1.0.0"
        self.options = None

    def parseOutputString(self, output):
        parser = NetsparkerXmlParser(output, self)
        host_names_resolve = {}
        for i in parser.items:

            if i.hostname not in host_names_resolve:
                ip = self.resolve_hostname(i.hostname)
                host_names_resolve[i.hostname] = ip
            else:
                ip = host_names_resolve[i.hostname]
            h_id = self.createAndAddHost(ip, hostnames=[i.hostname])

            s_id = self.createAndAddServiceToHost(h_id, str(i.port),
                                                       protocol = str(i.protocol),
                                                       ports=[str(i.port)],
                                                       status="open")

            if i.resolution is not None:
                resolution = BeautifulSoup(i.resolution, "lxml").text
            else:
                resolution = ""

            if i.desc is not None:
                desc = BeautifulSoup(i.desc, "lxml").text
            else:
                desc = ""
            if i.name_title is None:
                name = i.name
            else:
                name = i.name_title
            self.createAndAddVulnWebToService(h_id, s_id, name, ref=i.ref, website=i.hostname,
                                                     severity=i.severity, desc=desc, path=i.url, method=i.method,
                                                     request=i.request, response=i.response, resolution=resolution,
                                                     pname=i.param, data=i.data, cve=i.cve, cwe=i.cwe, cvss3=i.cvss3)

        del parser


def createPlugin(*args, **kwargs):
    return NetsparkerPlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2013  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information
"""
import re
import xml.etree.ElementTree as ET
from urllib.parse import urlparse

from faraday_plugins.plugins.plugin import PluginXMLFormat

__author__ = "Francisco Amato"
__copyright__ = "Copyright (c) 2013, Infobyte LLC"
__credits__ = ["Francisco Amato"]
__license__ = ""
__version__ = "1.0.0"
__maintainer__ = "Francisco Amato"
__email__ = "famato@infobytesec.com"
__status__ = "Development"


class ZapXmlParser:
    """
    The objective of this class is to parse an xml
    file generated by the zap tool.

    TODO: Handle errors.
    TODO: Test zap output version. Handle what happens
          if the parser doesn't support it.

    TODO: Test cases.

    @param zap_xml_filepath A proper xml generated by zap
    """

    def __init__(self, xml_output, resolve_hostname):
        self.resolve_hostname = resolve_hostname
        tree = self.parse_xml(xml_output)

        if tree is not None:
            self.sites = [data for data in self.get_items(tree)]
        else:
            self.sites = []

    @staticmethod
    def parse_xml(xml_output):
        """
        Open and parse an xml file.

        TODO: Write custom parser to just read the nodes that we need instead of
        reading the whole file.

        @return xml_tree An xml tree instance. None if error.
        """
        try:
            parser = ET.XMLParser(target=ET.TreeBuilder())
            parser.feed(xml_output)
            tree = parser.close()

        except SyntaxError as err:
            print(f"SyntaxError: {err}. {xml_output}")
            return None

        return tree

    def get_items(self, tree):
        """
        @return items A list of Host instances
        """
        for node in tree.findall('site'):
            yield Site(node, self.resolve_hostname)


def get_attrib_from_subnode(xml_node, subnode_xpath_expr, attrib_name):
    """
    Finds a subnode in the item node and the retrieves a value from it

    @return An attribute value
    """

    node = xml_node.find(subnode_xpath_expr)

    if node is not None:
        return node.get(attrib_name)

    return None


def strip_tags(data):
    """
    Remove html tags from a string
    @return Stripped string
    """
    clean = re.compile('<.*?>')
    return re.sub(clean, '', data)


class Site:

    def __init__(self, item_node, resolve_hostname):
        self.node = item_node

        self.host = self.node.get('host')
        self.ip = resolve_hostname(self.host)
        self.port = self.node.get('port')
        self.ssl = self.node.get('ssl')

        self.items = []
        for alert in self.node.findall('alerts/alertitem'):
            self.items.append(Item(alert))

    def get_text_from_subnode(self, subnode_xpath_expr):
        """
        Finds a subnode in the host node and the retrieves a value from it.

        @return An attribute value
        """
        sub_node = self.node.find(subnode_xpath_expr)
        if sub_node is not None:
            return sub_node.text
        return None


class Item:
    """
    An abstract representation of a Item


    @param item_node A item_node taken from an zap xml tree
    """

    def __init__(self, item_node):

        self.node = item_node
        self.id = self.get_text_from_subnode('pluginid')
        self.name = self.get_text_from_subnode('alert')
        self.severity = self.get_text_from_subnode('riskcode')
        self.desc = self.get_text_from_subnode('desc')
        if self.get_text_from_subnode('solution'):
            self.resolution = self.get_text_from_subnode('solution')
        else:
            self.resolution = ''
        self.cwe = []
        self.ref = []
        if self.get_text_from_subnode('reference'):
            links = self.get_text_from_subnode('reference')
            for link in links.split("</p>"):
                link = link.strip().replace("\n", "")
                if link != "":
                    self.ref.append(strip_tags(link))

        if self.get_text_from_subnode('cweid'):
            self.cwe.append("CWE-" + self.get_text_from_subnode('cweid'))

        if self.get_text_from_subnode('wascid'):
            self.ref.append("WASC:" + self.get_text_from_subnode('wascid'))

        self.items = []

        if item_node.find('instances'):
            arr = item_node.find('instances')
        else:
            arr = [item_node]

        for elem in arr:
            uri = elem.find('uri').text
            method = elem.findtext('method', "")
            item = self.parse_uri(uri, method)

            param = elem.findtext("param", "")
            attack = elem.findtext("attack", "")
            if attack and param:
                item["data"] = f"URL:\n {uri}\n Payload:\n {param} = {attack}"
            else:
                item["data"] = f"URL:\n {uri}\n Parameter:\n {param}"

            evidence = elem.findtext("evidence", "")
            if evidence:
                item["data"] = f"URL:\n {uri}\n Parameter:\n {param}\n Evidence:\n {evidence}"
            else:
                item["data"] = f"URL:\n {uri}\n"

            item["pname"] = elem.findtext("param", "")

            self.items.append(item)

    def parse_uri(self, uri, method) -> dict:

        parsed_url = urlparse(uri)
        protocol = parsed_url.scheme
        host = parsed_url.netloc
        port = parsed_url.port
        params = self.extract_params_from_uri(uri)

        return {
            'uri': uri,
            'params': ', '.join(params),
            'host': host,
            'website': f"{protocol}://{host}",
            'protocol': protocol,
            'port': port,
            'method': method,
            'path': parsed_url.path,
            'query': parsed_url.query,
            'data': ""
        }

    @staticmethod
    def extract_params_from_uri(uri):
        params = re.findall(r"(\w+)=", uri)
        return params if params else ''

    def get_text_from_subnode(self, subnode_xpath_expr):
        """
        Finds a subnode in the host node and the retrieves a value from it.

        @return An attribute value
        """
        sub_node = self.node.find(subnode_xpath_expr)
        if sub_node is not None:
            return sub_node.text

        return None


class ZapPlugin(PluginXMLFormat):
    """
    Example plugin to parse zap output.
    """

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.identifier_tag = "OWASPZAPReport"
        self.id = "Zap"
        self.name = "Zap XML Output Plugin"
        self.plugin_version = "0.0.4"
        self.version = "2.10.0"
        self.framework_version = "1.0.0"
        self.options = None

    def parseOutputString(self, output):
        """
        This method will discard the output the shell sends, it will read it
        from the xml where it expects it to be present.
        """

        parser = ZapXmlParser(output, self.resolve_hostname)

        for site in parser.sites:

            host = []
            if site.host != site.ip:
                host = [site.host]

            if site.ssl == "true":
                service = "https"
            else:
                service = "http"

            h_id = self.createAndAddHost(site.ip, hostnames=host)

            s_id = self.createAndAddServiceToHost(h_id, service, "tcp", ports=[site.port], status='open')

            for item in site.items:
                for instance in item.items:
                    self.createAndAddVulnWebToService(
                        h_id,
                        s_id,
                        item.name,
                        strip_tags(item.desc),
                        website=instance['website'],
                        query=instance['query'],
                        severity=item.severity,
                        path=instance['path'],
                        params=instance['params'],
                        method=instance['method'],
                        ref=item.ref,
                        resolution=strip_tags(item.resolution),
                        data=instance["data"],
                        pname=instance["pname"],
                        external_id="ZAP-" + str(item.id),
                        cwe=item.cwe
                    )

        del parser


def createPlugin(*args, **kwargs):
    return ZapPlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2017  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information
"""
import re

__author__ = "Roberto Focke"
__copyright__ = "Copyright (c) 2017, Infobyte LLC"
__license__ = ""
__version__ = "1.0.0"

from faraday_plugins.plugins.plugin import PluginBase


class xsssniper(PluginBase):

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.id = "xsssniper"
        self.name = "xsssniper"
        self.plugin_version = "0.0.1"
        self.version = "1.0.0"
        self.protocol = "tcp"
        self._command_regex = re.compile(r'^(sudo xsssniper|xsssniper|sudo xsssniper\.py|xsssniper\.py|sudo python'
                                         r'xsssniper\.py|.\/xsssniper\.py|python xsssniper\.py)\s+')

    def parseOutputString(self, output):
        parametro = []
        lineas = output.split("\n")
        aux = 0
        for linea in lineas:
            if not linea:
                continue
            linea = linea.lower()
            if (linea.find("target:")>0):
                url = re.findall(r'(?:[-\w.]|(?:%[\da-fA-F]{2}))+', linea)
                address = self.resolve_hostname(url[3])
                host_id = self.createAndAddHost(address, hostnames=url[3])
            if (linea.find("method")>0):
                list_a = re.findall(r"\w+", linea)
                metodo= list_a[1]
            if (linea.find("query string:")>0):
                lista_parametros=linea.split('=')
                aux=len(lista_parametros)
            if (linea.find("param:")>0):
                list2 = re.findall(r"\w+",linea)
                parametro.append(list2[1])
                service_id = self.createAndAddServiceToHost(host_id, self.protocol, 'tcp', ports=['80'], status='Open',
                                                            version="", description="")
        if aux != 0:
            self.createAndAddVulnWebToService(host_id, service_id, name="xss", desc="XSS", ref='', severity='med',
                                              website=url[0], path='', method=metodo, pname='',
                                              params=''.join(parametro), request='', response='')


def createPlugin(*args, **kwargs):
    return xsssniper(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2020  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information

"""
import json
from faraday_plugins.plugins.plugin import PluginJsonFormat

__author__ = "Blas Moyano"
__copyright__ = "Copyright (c) 2020, Infobyte LLC"
__credits__ = ["Blas Moyano"]
__license__ = ""
__version__ = "0.0.1"
__maintainer__ = "Blas Moyano"
__email__ = "bmoyano@infobytesec.com"
__status__ = "Development"


class WhatWebJsonParser:

    def __init__(self, json_output):
        list_data = json.loads(json_output)
        self.host_whatweb = []
        for info in list_data:
            try:
                server_info = info['plugins']['HTTPServer']
            except KeyError:
                server_info = {}

            try:
                ip_info = info['plugins']['IP']
            except KeyError:
                ip_info = {}

            try:
                country_info = info['plugins']['Country']
            except KeyError:
                country_info = {}

            whatweb_data = {
                "url": info.get('target', None),
                "os": None if not server_info else server_info.get('os', None),
                "os_detail": "Unknown" if not server_info else server_info.get('string', 'Unknown'),
                "ip": ['0.0.0.0'] if ip_info is None else ip_info.get('string', None),
                "country": "" if country_info is None else country_info.get('string', "")
            }
            self.host_whatweb.append(whatweb_data)


class WhatWebPlugin(PluginJsonFormat):

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.id = "whatweb"
        self.name = "WhatWebPlugin"
        self.plugin_version = "0.1"
        self.version = "0.0.1"
        self.json_keys = {'target', 'http_status', 'plugins'}


    def parseOutputString(self, output):
        parser = WhatWebJsonParser(output)
        for whatweb_data in parser.host_whatweb:
            desc = f"{whatweb_data['os_detail']} - {whatweb_data['country']}"
            if whatweb_data['os'] is None:
                datail_os = "Unknown"
            else:
                datail_os = whatweb_data['os'][0]

            self.createAndAddHost(whatweb_data['ip'][0],
                                  os=datail_os,
                                  hostnames=whatweb_data['url'],
                                  description=desc)


def createPlugin(*args, **kwargs):
    return WhatWebPlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2013  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information

"""

import xml.etree.ElementTree as ET

from dateutil.parser import parse
from faraday_plugins.plugins.plugin import PluginXMLFormat

__author__ = "Blas"
__copyright__ = "Copyright (c) 2019, Infobyte LLC"
__credits__ = ["Blas", "Nicolas Rebagliati"]
__license__ = ""
__version__ = "1.0.0"
__maintainer__ = "Blas"
__email__ = "bmoyano@infobytesec.com"
__status__ = "Development"

from faraday_plugins.plugins.repo.nessus.DTO import ReportHost, Report, ReportItem


class NessusParser:
    """
    The objective of this class is to parse an xml file generated by the nessus tool.

    TODO: Handle errors.
    TODO: Test nessus output version. Handle what happens if the parser doesn't support it.
    TODO: Test cases.

    @param nessus_filepath A proper simple report generated by nessus
    """

    def __init__(self, output):
        self.tree = ET.fromstring(output)
        self.report = []
        if self.tree:
            self.report = self.__get_report()

    def __get_report(self) -> Report:
        report = self.tree.find('Report')
        return Report(report) if report else None


class NessusPlugin(PluginXMLFormat):
    """
    Example plugin to parse nessus output.
    """

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.extension = ".nessus"
        self.identifier_tag = "NessusClientData_v2"
        self.id = "Nessus"
        self.name = "Nessus XML Output Plugin"
        self.plugin_version = "0.0.1"
        self.version = "5.2.4"
        self.framework_version = "1.0.1"
        self.options = None

    @staticmethod
    def parse_compliance_data(data: dict):
        compliance_data = {}
        for key, value in data.items():
            if 'compliance-' in key:
                compliance_name = key.split("}")[-1]
                compliance_data[compliance_name] = value
        return compliance_data

    def map_properties(self, host: ReportHost):
        if self.hostname_resolution:
            name = host.host_properties.host_ip if host.host_properties.host_ip else host.name
        else:
            name = host.name
        hostnames = [host.host_properties.host_fqdn]
        if host.host_properties.host_rdns and host.host_properties.host_rdns not in hostnames:
            hostnames.append(host.host_properties.host_rdns)
        return {
            "name": name,
            "hostnames": hostnames,
            "mac": host.host_properties.mac_address,
            "os": host.host_properties.operating_system
        }

    @staticmethod
    def map_item(host_id, run_date, plugin_name, item: ReportItem) -> dict:
        data = item.plugin_output
        data += f'{item.exploit_available}'
        return {
            "host_id": host_id,
            "name": plugin_name,
            "severity": item.risk_factor,
            "data": data,
            "external_id": item.plugin_id_attr,
            "run_date": run_date,
            "desc": item.description,
            "resolution": item.solution,
            "ref": [],
        }

    def map_policy_general(self, kwargs, item: ReportItem):
        kwargs.update({"policyviolations": []})
        if item.plugin_family_attr == 'Policy Compliance':
            data = item.get_data()
            bis_benchmark_data = kwargs["desc"].split('\n')
            compliance_data = self.parse_compliance_data(data)
            compliance_info = compliance_data.get('compliance-info', '')
            if compliance_info and not kwargs["desc"]:
                kwargs["desc"] = compliance_info
            compliance_reference = compliance_data.get(
                'compliance-reference', '').replace('|', ':').split(',')
            compliance_result = compliance_data.get('compliance-result', '')
            for reference in compliance_reference:
                kwargs["ref"].append(reference)
            compliance_check_name = compliance_data.get('compliance-check-name', '')
            compliance_solution = compliance_data.get('compliance-solution', '')
            if compliance_solution and not kwargs["resolution"]:
                kwargs["resolution"] = compliance_solution
            policy_item = f'{compliance_check_name} - {compliance_result}'
            for policy_check_data in bis_benchmark_data:
                if 'ref.' in policy_check_data:
                    kwargs["ref"].append(policy_check_data)
            if 'compliance-see-also' in compliance_data:
                kwargs["ref"].append(compliance_data.get('compliance-see-also'))
            # We used this info from tenable: https://community.tenable.com/s/article/Compliance-checks-in-SecurityCenter
            kwargs["policyviolations"].append(policy_item)
            kwargs["name"] = f'{kwargs["name"]}: {policy_item}'

        return kwargs

    def parseOutputString(self, output):
        """
        This method will discard the output the shell sends, it will read it from
        the xml where it expects it to be present.

        NOTE: if 'debug' is true then it is being run from a test case and the
        output being sent is valid.
        """

        try:
            parser = NessusParser(output)
        except Exception as e:
            self.logger.error(str(e))
            return None
        report_hosts = parser.report.report_hosts
        if report_hosts:
            for host in report_hosts:
                run_date = host.host_properties.host_end
                if run_date:
                    run_date = parse(run_date)
                website = host.host_properties.host_fqdn
                host_id = self.createAndAddHost(**self.map_properties(host))

                for item in host.report_items:
                    vulnerability_name = item.plugin_name
                    if not vulnerability_name:
                        continue
                    item_name = item.svc_name_attr

                    _main_data = self.map_item(
                        host_id, run_date, vulnerability_name, item)

                    _main_data = self.map_add_ref(_main_data, item)
                    if item_name == 'general':
                        _main_data = self.map_policy_general(_main_data, item)
                        self.createAndAddVulnToHost(**_main_data)
                    else:
                        _main_data["service_id"] = self.createAndAddServiceToHost(
                            host_id, name=item_name, protocol=item.protocol_attr,
                            ports=item.port_attr)
                        if item_name == 'www' or item_name == 'http':
                            _main_data.update({"website": website})
                            self.createAndAddVulnWebToService(**_main_data)
                        else:
                            self.createAndAddVulnToService(**_main_data)

    @staticmethod
    def map_add_ref(kwargs, item: ReportItem):
        kwargs["cvss2"] = {}
        kwargs["cvss3"] = {}
        if item.see_also:
            kwargs["ref"].append(item.see_also)
        if item.cpe:
            kwargs["ref"].append(item.cpe)
        if item.cve:
            kwargs["cve"] = item.cve
        if item.cwe:
            kwargs["cwe"] = item.cwe
        if item.cvss3_vector:
            kwargs["cvss3"]["vector_string"] = item.cvss3_vector
        if item.cvss_vector:
            kwargs["cvss2"]["vector_string"] = item.cvss_vector
        return kwargs


def createPlugin(*args, **kwargs):
    return NessusPlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2013  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information

"""
import re
from faraday_plugins.plugins.plugin import PluginBase

__author__ = "Ezequiel Tavella - @EzequielTBH"
__copyright__ = "Copyright 2015, @EzequielTBH"
__credits__ = "Ezequiel Tavella - @EzequielTBH"
__license__ = "GPL v3"
__version__ = "1.0.0"


class traceroutePlugin(PluginBase):

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.id = "Traceroute"
        self.name = "Traceroute"
        self.plugin_version = "1.0.0"
        self.command_string = ""
        self._command_regex = re.compile(r'^(traceroute|traceroute6)\s+.*?')

    def parseOutputString(self, output):

        print("[*]Parsing Output...")

        # Check no results.
        if not output.startswith("traceroute to"):
            return

        # Check if last parameter is host or ( packetlen or data size).
        parameters = self.command_string.split(' ')
        parameters.reverse()
        hostName = parameters[0]

        try:
            int(hostName)
            # No exception => host is the next item.
            hostName = parameters[1]
        except:
            pass

        # Add host and note with output of traceroute.
        hostId = self.createAndAddHost(hostName)
        self.createAndAddNoteToHost(hostId, "Traceroute Results", output)

        print("[*]Parse finished, API faraday called...")

    def processCommandString(self, username, current_path, command_string):
        super().processCommandString(username, current_path, command_string)
        self.command_string = command_string
        return None


def createPlugin(*args, **kwargs):
    return traceroutePlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2013  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information

"""
import re

from faraday_plugins.plugins.plugin import PluginBase


__author__ = "Francisco Amato"
__copyright__ = "Copyright (c) 2013, Infobyte LLC"
__credits__ = ["Francisco Amato"]
__license__ = ""
__version__ = "1.0.0"
__maintainer__ = "Francisco Amato"
__email__ = "famato@infobytesec.com"
__status__ = "Development"

valid_records = ["NS", "CNAME", "A"]


class FierceParser:
    """
    The objective of this class is to parse an shell output generated by
    the fierce tool.

    TODO: Handle errors.
    TODO: Test fierce output version. Handle what happens if the parser
    doesn't support it.
    TODO: Test cases.

    @param fierce_filepath A proper simple report generated by fierce
    """

    def __init__(self, output):
        self.target = None
        self.items = []

        regex = re.search(
            "DNS Servers for ([\\w\\.-]+):\n([^$]+)Trying zone transfer first...",
            output)

        if regex is not None:
            self.target = regex.group(1)
            mstr = re.sub("\t", "", regex.group(2))
            self.dns = list(filter(None, mstr.splitlines()))

        regex = re.search(
            "Now performing [\\d]+ test\\(s\\)...\n([^$]+)\nSubnets found ",
            output)
        if regex is not None:
            hosts_list = regex.group(1).splitlines()
            for i in hosts_list:
                if i != "":
                    mstr = i.split("\t")
                    host = mstr[1]
                    record = "A"
                    ip = mstr[0]
                    self.add_host_info_to_items(ip, host, record)

        self.isZoneVuln = False
        output = output.replace('\\$', '')
        regex = re.search(
            "Whoah, it worked - misconfigured DNS server found:([^$]+)\nThere isn't much point continuing, "
            "you have  everything.", output)

        if regex is not None:
            self.isZoneVuln = True
            dns_list = regex.group(1).splitlines()
            for i in dns_list:
                if i != "":
                    mstr = i.split()
                    if (mstr and mstr[0] != "" and len(mstr) > 3 and mstr[3] in valid_records):
                        host = mstr[0]
                        record = mstr[3]
                        ip = mstr[4]
                        self.add_host_info_to_items(ip, host, record)

    def add_host_info_to_items(self, ip_address, hostname, record):
        data = {}
        exists = False
        for item in self.items:
            if ip_address in item['ip']:
                item['hosts'].append(hostname)
                exists = True

        if not exists:
            data['ip'] = ip_address
            data['hosts'] = [hostname]
            data['record'] = record
            self.items.append(data)


class FiercePlugin(PluginBase):
    """
    Example plugin to parse fierce output.
    """

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.id = "Fierce"
        self.name = "Fierce Output Plugin"
        self.plugin_version = "0.0.1"
        self.version = "0.9.9"
        self.options = None
        self._current_output = None
        self._command_regex = re.compile(
            r'^(sudo fierce|fierce|sudo fierce\.pl|fierce\.pl|perl fierce\.pl|\.\/fierce\.pl)\s+.*?')


    def resolveCNAME(self, item, items):
        for i in items:
            if (item['ip'] in i['hosts']):
                item['ip'] = i['ip']
                return item
        try:
            item['ip'] = self.resolve_hostname(item['ip'])
        except:
            pass  # nosec
        return item

    def resolveNS(self, item, items):
        try:
            item['hosts'][0] = item['ip']
            item['ip'] = self.resolve_hostname(item['ip'])
        except:
            pass
        return item

    def parseOutputString(self, output):

        parser = FierceParser(output)
        for item in parser.items:

            item['isResolver'] = False
            item['isZoneVuln'] = False
            if item['record'] == "CNAME":
                self.resolveCNAME(item, parser.items)
            if item['record'] == "NS":
                self.resolveNS(item, parser.items)
                item['isResolver'] = True
                item['isZoneVuln'] = parser.isZoneVuln
                for item2 in parser.items:

                    if item['ip'] == item2['ip'] and item != item2:
                        item2['isResolver'] = item['isResolver']
                        item2['isZoneVuln'] = item['isZoneVuln']
                        item['ip'] = ''

        for item in parser.items:
            if item['ip'] == "127.0.0.1" or item['ip'] == '':
                continue
            h_id = self.createAndAddHost(
                item['ip'],
                hostnames=item['hosts'])

            if item['isResolver']:
                s_id = self.createAndAddServiceToHost(
                    h_id,
                    "domain",
                    "tcp",
                    ports=['53'])

                if item['isZoneVuln']:
                    self.createAndAddVulnToService(
                        h_id,
                        s_id,
                        "Zone transfer",
                        desc="A Dns server allows unrestricted zone transfers",
                        ref=["CVE-1999-0532"])


def createPlugin(*args, **kwargs):
    return FiercePlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2013  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information
"""
import re

__author__ = "Federico Fernandez - @q3rv0"
__copyright__ = "Copyright (c) 2013, Infobyte LLC"
__license__ = ""
__version__ = "1.0.0"
__maintainer__ = "Federico Fernandez"
__email__ = "fede.merlo26@gmail.com"
__status__ = "Development"

from faraday_plugins.plugins.plugin import PluginBase


class dirbPlugin(PluginBase):

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.id = "dirb"
        self.name = "Dirb"
        self.plugin_version = "0.0.1"
        self.version = "2.22"
        self.regexpUrl = r'((http[s]?)\:\/\/([\w\.]+)[.\S]+)'
        self._command_regex = re.compile(r'^(sudo dirb|dirb|\.\/dirb|sudo \.\/dirb)\s+.*?')

        self.text = []

    def getPort(self, host, proto):
        p = re.search(r"\:([0-9]+)\/", host)
        if p is not None:
            return p.group(1)
        elif proto == 'https':
            return 443
        else:
            return 80

    def getIP(self, host):
        try:
            ip = self.resolve_hostname(host)
        except Exception:  # nosec
            pass

        return ip

    def state(self, output):
        if output.find('COULDNT CONNECT') != -1:
            return "close"
        else:
            return "open"

    def pathsDirListing(self, output):
        data = []
        r = re.findall(self.regexpUrl + r"[\-\._\w\*\s]+\s+\(!\) WARNING: Directory IS LISTABLE",
                        output)
        for u in r:
            data.append(u[0])

        paths = "\n".join(data)
        return paths

    def note(self, output):
        dirs  = re.findall(r"==> DIRECTORY: "+self.regexpUrl, output)
        files = re.findall(r"\+ " + self.regexpUrl + r" \(.+\)", output)
        for d in dirs:
            self.text.append("DIRECTORY: " + d[0])

        for f in files:
            self.text.append("FILE: " + f[0])

        self.text = '\n'.join(self.text)

    def parseOutputString(self, output):

        url = re.search(r"URL_BASE: " + self.regexpUrl, output)
        paths = self.pathsDirListing(output)
        status = self.state(output)
        self.note(output)

        if output.find('END_TIME') != -1 and url is not None:
            proto = url.group(2)
            domain = url.group(3)
            ip = self.getIP(domain)
            puerto = self.getPort(url.group(1), proto)

            host_id = self.createAndAddHost(ip)
            serv_id = self.createAndAddServiceToHost(host_id, proto, protocol=proto, ports=[puerto], status=status)
            if len(self.text) > 0:
                self.createAndAddVulnWebToService(host_id, serv_id, 'Url Fuzzing', severity=0, desc=self.text,
                                                  website=domain)
            if len(paths) > 0:
                self.createAndAddVulnWebToService(host_id, serv_id, "Directory Listing", severity="med", website=domain,
                                                  request=paths, method="GET")
        return True

    def processCommandString(self, username, current_path, command_string):
        """
        Adds the -oX parameter to get xml output to the command string that the
        user has set.
        """
        super().processCommandString(username, current_path, command_string)
        no_stop_on_warn_msg_re = r"\s+-w"
        arg_search = re.search(no_stop_on_warn_msg_re,command_string)
        extra_arg = ""
        if arg_search is None:
            extra_arg +=" -w"

        silent_mode_re = r"\s+-S"
        arg_search = re.search(silent_mode_re,command_string)
        if arg_search is None:
            extra_arg +=" -S"
        return f"{command_string}{extra_arg}"


def createPlugin(*args, **kwargs):
    return dirbPlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2013  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information

"""
from urllib.parse import urlsplit
from bs4 import BeautifulSoup
from lxml import etree

from faraday_plugins.plugins.plugin import PluginXMLFormat
from faraday_plugins.plugins.repo.invicti.DTO import Invicti

__author__ = "Gonzalo Martinez"
__copyright__ = "Copyright (c) 2013, Infobyte LLC"
__credits__ = ["Gonzalo Martinez"]
__version__ = "1.0.0"
__maintainer__ = "Gonzalo Martinez"
__email__ = "gmartinez@infobytesec.com"
__status__ = "Development"


class InvictiXmlParser:
    """
    The objective of this class is to parse a xml file generated by
    the acunetix tool.

    @param invicti_xml_filepath A proper xml generated by acunetix
    """

    def __init__(self, xml_output):

        tree = self.parse_xml(xml_output)
        self.invicti = Invicti(tree)

    @staticmethod
    def parse_xml(xml_output):
        """
        Open and parse an xml file.

        TODO: Write custom parser to just read the nodes that we need instead
        of reading the whole file.

        @return xml_tree An xml tree instance. None if error.
        """

        try:
            parser = etree.XMLParser(recover=True)
            tree = etree.fromstring(xml_output, parser=parser)
        except SyntaxError as err:
            print(f"SyntaxError: {err}. {xml_output}")
            return None

        return tree


class InvictiPlugin(PluginXMLFormat):
    """
    Example plugin to parse invicti output.
    """

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.identifier_tag = "invicti-enterprise"
        self.id = "Invicti"
        self.name = "Invicti XML Output Plugin"
        self.plugin_version = "1.0.0"
        self.version = "9"
        self.framework_version = "1.0.0"
        self.options = None
        self._current_output = None
        self.target = None

    def parseOutputString(self, output):
        """
        This method will discard the output the shell sends, it will read it
        from the xml where it expects it to be present.

        NOTE: if 'debug' is true then it is being run from a test case and the
        output being sent is valid.
        """
        parser = InvictiXmlParser(output)
        url = urlsplit(parser.invicti.target.url)
        ip = self.resolve_hostname(url.netloc)
        h_id = self.createAndAddHost(ip)
        s_id = self.createAndAddServiceToHost(h_id, url.scheme, ports=433)
        for vulnerability in parser.invicti.vulnerabilities:
            vuln = {"name": vulnerability.name, "severity": vulnerability.severity,
                    "confirmed": vulnerability.confirmed,
                    "desc": BeautifulSoup(vulnerability.description, features="lxml").text,
                    "path": vulnerability.url.replace(parser.invicti.target.url, ""),
                    "external_id": vulnerability.look_id,
                    "resolution": BeautifulSoup(vulnerability.remedial_procedure, features="lxml").text}
            if vulnerability.classification:
                references = []
                if vulnerability.classification.owasp:
                    references.append("OWASP" + vulnerability.classification.owasp)
                if vulnerability.classification.wasc:
                    references.append("WASC" + vulnerability.classification.wasc)
                if vulnerability.classification.cwe:
                    vuln["cwe"] = "CWE-" + vulnerability.classification.cwe
                if vulnerability.classification.capec:
                    references.append("CAPEC" + vulnerability.classification.capec)
                if vulnerability.classification.pci32:
                    references.append("PCI32" + vulnerability.classification.pci32)
                if vulnerability.classification.hipaa:
                    references.append("HIPAA" + vulnerability.classification.hipaa)
                if vulnerability.classification.owasppc:
                    references.append("OWASPPC" + vulnerability.classification.owasppc)
                if vulnerability.classification.cvss3.node is not None:
                    vuln["cvss3"] = {"vector_string": vulnerability.classification.cvss3.vector}
                vuln["ref"] = references
            if vulnerability.http_response.node is not None:
                vuln["response"] = vulnerability.http_response.content
            if vulnerability.http_request.node is not None:
                vuln["request"] = vulnerability.http_request.content
            self.createAndAddVulnWebToService(h_id, s_id, **vuln)


def createPlugin(*args, **kwargs):
    return InvictiPlugin(*args, **kwargs)



from typing import List
import re

class Technicaldetails:
    def __init__(self, node):
        self.node = node

    @property
    def request(self) -> str:
        if self.node in ('', None):
            return ''
        return self.node.findtext('Request', '')

    @property
    def response(self) -> str:
        if self.node in ('', None):
            return ''
        return self.node.findtext('Response', '')


class Cve:
    def __init__(self, node):
        self.node = node

    @property
    def text(self) -> str:
        if self.node in ('', None):
            return ''
        return self.node.text


class CVEList:
    def __init__(self, node):
        self.node = node

    @property
    def cve(self) -> Cve:
        if self.node is None:
            return ''
        return Cve(self.node.find('CVE'))


class Cwe:
    def __init__(self, node):
        self.node = node

    @property
    def id_attr(self) -> str:
        return self.node.attrib.get('id', '')

    @property
    def text(self) -> str:
        return self.node.text


class Cwelist:
    def __init__(self, node):
        self.node = node

    @property
    def cwe(self) -> Cwe:
        return Cwe(self.node.find('CWE'))


class Cvss:
    def __init__(self, node):
        self.node = node

    @property
    def descriptor(self) -> str:
        return self.node.findtext('Descriptor', '')

    @property
    def score(self) -> str:
        if self.node is None:
            return ''
        return self.node.findtext('Score')

    @property
    def av(self) -> str:
        return self.node.findtext('AV', '')

    @property
    def ac(self) -> str:
        return self.node.findtext('AC', '')

    @property
    def au(self) -> str:
        return self.node.findtext('Au', '')

    @property
    def c(self) -> str:
        return self.node.findtext('C', '')

    @property
    def i(self) -> str:
        return self.node.findtext('I', '')

    @property
    def a(self) -> str:
        return self.node.findtext('A', '')

    @property
    def e(self):
        return self.node.find('E')

    @property
    def rl(self):
        return self.node.find('RL')

    @property
    def rc(self):
        return self.node.find('RC')


class Cvss3:
    def __init__(self, node):
        self.node = node

    @property
    def descriptor(self) -> str:
        return self.node.findtext('Descriptor', '')

    @property
    def score(self) -> str:
        return self.node.findtext('Score')

    @property
    def tempscore(self):
        return self.node.find('TempScore')

    @property
    def envscore(self):
        return self.node.find('EnvScore')

    @property
    def av(self) -> str:
        return self.node.find('AV', '')

    @property
    def ac(self) -> str:
        return self.node.find('AC', '')

    @property
    def pr(self) -> str:
        return self.node.find('PR', '')

    @property
    def ui(self) -> str:
        return self.node.find('UI', '')

    @property
    def s(self) -> str:
        return self.node.find('S', '')

    @property
    def c(self) -> str:
        return self.node.find('C', '')

    @property
    def i(self) -> str:
        return self.node.findtext('I', '')

    @property
    def a(self) -> str:
        return self.node.findtext('A', '')

    @property
    def e(self):
        return self.node.find('E')

    @property
    def rl(self):
        return self.node.find('RL')

    @property
    def rc(self):
        return self.node.find('RC')


class Reference:
    def __init__(self, node):
        self.node = node

    @property
    def database(self) -> str:
        return self.node.findtext('Database', '')

    @property
    def url(self) -> str:
        return self.node.findtext('URL', '')


class References:
    def __init__(self, node):
        self.node = node

    @property
    def reference(self) -> List[Reference]:
        return [Reference(i) for i in self.node.findall('Reference', [])]


class Reportitem:
    def __init__(self, node):
        self.node = node

    @property
    def id_attr(self) -> str:
        return self.node.findtext('id', '')

    @property
    def color_attr(self) -> str:
        return self.node.findtext('color', '')

    @property
    def name(self) -> str:
        return self.node.findtext('Name', '')

    @property
    def modulename(self) -> str:
        return self.node.findtext('ModuleName', '')

    @property
    def details(self) -> str:
        return self.node.findtext('Details', '')

    @property
    def affects(self) -> str:
        return self.node.findtext('Affects', '')

    @property
    def parameter(self) -> str:
        return self.node.findtext('Parameter')

    @property
    def aop_sourcefile(self):
        return self.node.find('AOP_SourceFile')

    @property
    def aop_sourceline(self):
        return self.node.find('AOP_SourceLine')

    @property
    def aop_additional(self):
        return self.node.find('AOP_Additional')

    @property
    def isfalsepositive(self):
        return self.node.find('IsFalsePositive')

    @property
    def severity(self) -> str:
        return self.node.findtext('Severity', '')

    @property
    def type(self) -> str:
        return self.node.findtext('Type', '')

    @property
    def impact(self) -> str:
        return self.node.findtext('Impact', '')

    @property
    def description(self) -> str:
        return self.node.findtext('Description', '')

    @property
    def recommendation(self) -> str:
        return self.node.findtext('Recommendation', '')

    @property
    def technicaldetails(self) -> Technicaldetails:
        return Technicaldetails(self.node.find('TechnicalDetails'))

    @property
    def cwelist(self) -> Cwelist:
        return Cwelist(self.node.find('CWEList'))

    @property
    def cvelist(self):
        return CVEList(self.node.find('CVEList'))

    @property
    def cvss(self) -> Cvss:
        return Cvss(self.node.find('cvss'))

    @property
    def cvss3(self) -> Cvss3:
        return Cvss3(self.node.find('cvss3'))

    @property
    def references(self) -> References:
        return References(self.node.find('References'))


class Reportitems:
    def __init__(self, node):
        self.node = node

    @property
    def reportitem(self) -> List[Reportitem]:
        return [Reportitem(i) for i in self.node.findall('ReportItem', [])]


class Crawler:
    def __init__(self, node):
        self.node = node

    @property
    def start_url_attr(self) -> str:
        return self.node.get('StartUrl', '')


class Scan:
    def __init__(self, node):
        self.node = node

    @property
    def reportitems(self) -> Reportitems:
        return Reportitems(self.node.find('ReportItems'))

    @property
    def start_url(self) -> str:
        return self.node.findtext("StartURL", "")

    @property
    def crawler(self) -> Crawler:
        return Crawler(self.node.find('Crawler'))

    @property
    def os(self) -> str:
        if not self.node.findtext("Os", "unknown"):
            return "unknown"
        return self.node.findtext("Os", "unknown")

    @property
    def banner(self) -> str:
        if not self.node.findtext('Banner'):
            return None
        return self.node.findtext("Banner")

    @property
    def start_url_new(self) -> str:
        return self.node.findtext("", "")


class Acunetix:
    def __init__(self, node):
        self.node = node

    @property
    def exportedon_attr(self) -> str:
        return self.node.get('ExportedOn')

    @property
    def scan(self) -> List[Scan]:
        return [Scan(i) for i in self.node.findall('Scan', [])]



"""
Faraday Penetration Test IDE
Copyright (C) 2013  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information

"""
import re
import os

from faraday_plugins.plugins.plugin import PluginBase


__author__ = "Facundo de Guzmán, Esteban Guillardoy"
__copyright__ = "Copyright (c) 2013, Infobyte LLC"
__credits__ = ["Facundo de Guzmán", "Esteban Guillardoy"]
__license__ = ""
__version__ = "1.0.0"
__maintainer__ = "Federico Kirschbaum"
__email__ = "fedek@infobytesec.com"
__status__ = "Development"


class CmdWhoisPlugin(PluginBase):
    """
    This plugin handles whois command.
    Basically detects if user was able to connect to a device
    """

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.id = "whois"
        self.name = "Whois"
        self.plugin_version = "0.0.1"
        self.version = "5.0.20"
        self.framework_version = "1.0.0"
        self.options = None
        self._current_output = None
        self._command_regex = re.compile(r'^whois\s+.*?')
        self._host_ip = None
        self._info = 0
        self._completition = {
            "": "whois [OPTION]... OBJECT...",
            "-l": "one level less specific lookup [RPSL only]",
            "-L": "find all Less specific matches",
            "-m": "find first level more specific matches",
            "-M": "find all More specific matches",
            "-c": "find the smallest match containing a mnt-irt attribute",
            "-x": "exact match [RPSL only]",
            "-d": "return DNS reverse delegation objects too [RPSL only]",
            "-i": "-i ATTR[,ATTR]...      do an inverse lookup for specified ATTRibutes",
            "-T": "-T TYPE[,TYPE]...      only look for objects of TYPE",
            "-K": "only primary keys are returned [RPSL only]",
            "-r": "turn off recursive lookups for contact information",
            "-R": "force to show local copy of the domain object even if it contains referral",
            "-a": "search all databases",
            "-s": "-s SOURCE[,SOURCE]...  search the database from SOURCE",
            "-g": "-g SOURCE:FIRST-LAST   find updates from SOURCE from serial FIRST to LAST",
            "-t": "-t TYPE request template for object of TYPE",
            "-v": "-v TYPE request verbose template for object of TYPE",
            "-q": "-q [version|sources|types]  query specified server info [RPSL only]",
            "-F": "fast raw output (implies -r)",
            "-h": "-h HOST connect to server HOST",
            "-p": "-p PORT connect to PORT",
            "-H": "hide legal disclaimers",
            "--verbose": "explain what is being done",
            "--help": "display this help and exit",
            "--version": "output version information and exit",
        }

    def processCommandString(self, username, current_path, command_string):
        self.command_string = command_string
        super().processCommandString(username, current_path, command_string)

    def parseOutputString(self, output):
        matches = re.findall(r"Name Server:\s*(.*)\s*", output)
        if not matches:
            matches = re.findall(r"nserver:\s*(.*)\s*", output)

        if not matches:
            ip = re.findall(r'[0-9]+(?:\.[0-9]+){3}', self.command_string)
            if not ip:
                url = self.command_string.replace('whois ', '')
                ip = [self.resolve_hostname(url)]
            matches_descr = re.findall(r"descr:\s*(.*)\s*", output)

            matches_netname = re.findall(r"NetName:\s*(.*)\s*", output)
            if not matches_netname:
                matches_netname = re.findall(r"netname:\s*(.*)\s*", output)
            matches_ref = re.findall(r"Ref:\s*(.*)\s*", output)
            desc = ""
            ref = []
            os_name = "unknown"

            for md in matches_descr:
                desc = md.strip()

            for mr in matches_ref:
                ref.append(mr.strip())

            for osname in matches_netname:
                os_name = osname.strip()
            self.createAndAddHost(
                ip[0],
                os_name,
                hostnames=[ref],
                description=desc
            )
        else:
            for m in matches:
                m = m.strip()
                url = re.findall(r'https?://[^\s<>"]+|.[^\s<>"]+', str(m))
                ip = self.resolve_hostname(url[0])
                self.createAndAddHost(
                    ip,
                    hostnames=[m]
                )
            matches_domain = re.findall(r"Domain Name:\s*(.*)\s*", output)
            for md in matches_domain:
                md = md.strip()
                ip = self.resolve_hostname(md)
                self.createAndAddHost(
                    ip,
                    hostnames=[md]
                )
        return True


def createPlugin(*args, **kwargs):
    return CmdWhoisPlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2013  Infobyte LLC (https://www.faradaysec.com/)
See the file 'doc/LICENSE' for the license information

"""
# Standard library imports
import logging
import os
import socket
import re
from urllib.parse import urlsplit

SERVICE_MAPPER = None
CVE_regex = re.compile(r'CVE-\d{4}-\d{4,7}')
CWE_regex = re.compile(r'CWE-\d{1,4}')
logger = logging.getLogger(__name__)


def get_vulnweb_url_fields(url):
    """Given a URL, return kwargs to pass to createAndAddVulnWebToService."""
    parse = urlsplit(url)
    return {
        "website": f"{parse.scheme}://{parse.netloc}",
        "path": parse.path,
        "query": parse.query
    }


def filter_services():
    global SERVICE_MAPPER
    if not SERVICE_MAPPER:
        logger.debug("Load service mappers")
        filename = os.path.join(os.path.dirname(__file__), "port_mapper.txt")
        with open(filename, encoding='utf-8') as fp:
            SERVICE_MAPPER = list(map(lambda x: x.strip().split('\t'), list(filter(len, fp.readlines()))))
    return SERVICE_MAPPER


def get_all_protocols():
    protocols = [
        'ip',
        'tcp',
        'udp',
        'icmp',
        'sctp',
        'hopopt',
        'igmp',
        'ggp',
        'ip-encap',
        'st',
        'egp',
        'igp',
        'pup',
        'hmp',
        'xns-idp',
        'rdp',
        'iso-tp4',
        'dccp',
        'xtp',
        'ddp',
        'idpr-cmtp',
        'ipv6',
        'ipv6-route',
        'ipv6-frag',
        'idrp',
        'rsvp',
        'gre',
        'ipsec-esp',
        'ipsec-ah',
        'skip',
        'ipv6-icmp',
        'ipv6-nonxt',
        'ipv6-opts',
        'rspf cphb',
        'vmtp',
        'eigrp',
        'ospfigp',
        'ax.25',
        'ipip',
        'etherip',
        'encap',
        'pim',
        'ipcomp',
        'vrrp',
        'l2tp',
        'isis',
        'fc',
        'udplite',
        'mpls-in-ip',
        'hip',
        'shim6',
        'wesp',
        'rohc',
        'mobility-header'
    ]

    yield from protocols


def get_severity_from_cvss(cvss):
    try:
        if type(cvss) != float:
            cvss = float(cvss)

        cvss_ranges = [(0.0, 0.1, 'info'),
                       (0.1, 4.0, 'low'),
                       (4.0, 7.0, 'med'),
                       (7.0, 9.0, 'high'),
                       (9.0, 10.1, 'critical')]
        for (lower, upper, severity) in cvss_ranges:
            if lower <= cvss < upper:
                return severity
    except ValueError:
        return 'unclassified'


def its_cve(cves: list):
    r = [cve for cve in cves if CVE_regex.match(cve)]
    return r


def its_cwe(cwes: list):
    r = [cwe for cwe in cwes if CWE_regex.match(cwe)]
    return r



"""
Faraday Penetration Test IDE
Copyright (C) 2013  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information

"""

from faraday_plugins.plugins.plugin import PluginCSVFormat
from itertools import islice
import csv
from dateutil.parser import parse

__author__ = "Erodriguez"
__copyright__ = "Copyright (c) 2019, Infobyte LLC"
__credits__ = ["Erodriguez"]
__license__ = ""
__version__ = "1.0.0"
__maintainer__ = "Erodriguez"
__email__ = "erodriguez@infobytesec.com"
__status__ = "Development"


class Appscan_CSV_Plugin(PluginCSVFormat):
    """
    Example plugin to parse Appscan output.
    """

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.csv_headers = {'HCL AppScan on Cloud'}
        self.id = "Appscan_CSV"
        self.name = "Appscan CSV Output Plugin"
        self.plugin_version = "0.0.1"
        self.version = "0.0.1"
        self.framework_version = "1.0.1"

    def _parse_filename(self, filename):
        with open(filename) as output:
            self.parseOutputString(islice(output, 15, None))

    def parseOutputString(self, output):
        try:
            reader = csv.DictReader(output)
        except:
            print("Error parser output")
            return None

        for row in reader:
            #Skip Fix Group
            if row["Issue Id"] == "Fix Group Attributes:":
                break
            path = row['Source File']
            if path == "":
                path = row['Location']
            try:
                run_date = parse(row['Date Created'])
            except:
                run_date = None
            name = row["Issue Type Name"]
            references = []
            if row["Cwe"]:
                references.append(f"CWE-{row['Cwe']}")
            if row["Cve"]:
                references.append(row["Cve"])

            data = []
            if row['Security Risk']:
                data.append(f"Security Risk: {row['Security Risk']}")
            desc = [row['Description']]
            if row['Cve']:
                desc.append(f"Cve:  {row['Cve']}")
            if row['Line']:
                desc.append(f"Line:  {row['Line']}")
            if row['Cause']:
                desc.append(f"Cause:  {row['Cause']}")
            if row['Remediation']:
                desc.append(f"Resolution:  {row['Resolution']}")
            if row['Threat Class']:
                desc.append(f"Threat Class:   {row['Threat Class']}")
            if row['Security Risk']:
                desc.append(f"Security Risk:   {row['Security Risk']}")
            if row['Calling Method']:
                desc.append(f"Calling Method:   {row['Calling Method']}")
            if row['Location']:
                desc.append(f"Vulnerability Line:   {row['Location']}")

            h_id = self.createAndAddHost(name=path)
            self.createAndAddVulnToHost(
                h_id,
                name=name,
                desc=" \n".join(desc),
                resolution=row['Remediation'],
                external_id=row['Issue Id'],
                cve=row['Cve'],
                run_date=run_date,
                severity=row["Severity"],
                ref=references,
                data=" \n".join(data)
            )

def createPlugin(*args, **kargs):
    return Appscan_CSV_Plugin(*args, **kargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2015  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information
"""
from faraday_plugins.plugins.plugin import PluginByExtension
import re
from datetime import datetime

__author__ = "Blas Moyano"
__copyright__ = "Copyright (c) 2020, Infobyte LLC"
__credits__ = ["Blas Moyano"]
__license__ = ""
__version__ = "1.0.0"
__maintainer__ = "Blas Moyano"
__status__ = "Development"


class MbsaParser:
    def __init__(self, log_output):
        self.computer_name = re.search('(Computer name:) (.*[A-Z])', log_output)
        self.ip = re.search(r'(IP address:) ([0-9]+(?:\.[0-9]+){3})', log_output)
        self.scan_date = re.search('(Scan date:) (.*[0-9])', log_output)
        self.issues = re.findall(r'Issue: .*', log_output)
        self.score = re.findall(r'Score: .*', log_output)
        self.result = re.findall(r'Result: .*', log_output)


class MbsaPlugin(PluginByExtension):

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.id = "MBSA"
        self.name = "Microsoft Baseline Security Analyzer"
        self.plugin_version = "1.0.1"
        self.version = "MBSA 1.0"
        self.framework_version = "1.0.0"
        self.extension = ".log"

    def parseOutputString(self, output):
        parser = MbsaParser(output)
        detail = ''
        i = 0
        issues_top = len(parser.issues)
        ip = '0.0.0.0'
        hostname = []
        run_date = None

        if parser.ip is not None:
            ip = parser.ip.group(2)
        if parser.computer_name is not None:
            hostname.append(parser.computer_name.group(2))
        if parser.scan_date is not None:
            run_date = datetime.strptime(parser.scan_date.group(2), '%Y/%m/%d %H:%M')

        host_id = self.createAndAddHost(
            ip,
            'Windows',
            hostnames=hostname)

        for issue in parser.issues:

            test = re.search(parser.issues[i], output)

            if i+1 != issues_top:
                test_issue = re.search(parser.issues[i+1], output)
            else:
                end = None
            try:
                start = test.end()
                end = test_issue.start()
            except:
                start = None

            if start is not None:
                if end is None:
                    result_info = output[start:]
                else:
                    result_info = output[start:end]
                    result_info.rstrip('\n')
                    result_info = result_info.replace(parser.score[i], '')
                    result_info = result_info.replace(parser.result[i], '')
                    result_info = result_info.strip()
                    if result_info:
                        detail = re.search('(Detail:)', result_info)
                        if not None:
                            detail = result_info
                            result_info = parser.result[i]

                    else:
                        detail = ''
                        result_info = parser.result[i]
            score = parser.score[i].replace('Score: ', '').strip()
            if score != 'Check passed':
                if score == 'Best practice' or score == 'Unable to scan':
                    severity = "info"
                elif score == 'Check failed (non-critical)':
                    severity = 'med'
                elif score == 'Check failed':
                    severity = 'high'
                else:
                    severity = 'info'

                self.createAndAddVulnToHost(
                    host_id,
                    issue.replace('Issue: ', '').strip(),
                    desc=result_info.replace('Result: ', '').strip(),
                    ref=None,
                    severity=severity,
                    data=detail,
                    run_date=run_date
                )

            i += 1


def createPlugin(*args, **kwargs):
    return MbsaPlugin(*args, **kwargs)



from setuptools import setup, find_packages
from re import search

with open('faraday_plugins/__init__.py', encoding='utf8') as f:
    version = search(r'__version__ = \'(.*?)\'', f.read()).group(1)


install_requires = [
    'Click',
    'simplejson',
    'requests',
    'lxml',
    'html2text',
    'beautifulsoup4',
    'pytz',
    'python-dateutil',
    'colorama',
    'tabulate',
    'packaging'
]


setup(
    name='faraday-plugins',
    version=version,
    packages=find_packages(include=['faraday_plugins', 'faraday_plugins.*']),
    url='',
    license="GNU General Public License v3",
    author='Faradaysec',
    author_email='devel@faradaysec.com',
    description='Faraday plugins package',
    include_package_data=True,
    install_requires=install_requires,
    entry_points={
            'console_scripts': [
                'faraday-plugins=faraday_plugins.commands:cli',
            ],
        },
)



import os
import re
from io import BytesIO

"""
Faraday Penetration Test IDE
Copyright (C) 2013  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information

"""
from lxml import etree
from lxml.etree import XMLParser
from faraday_plugins.plugins.plugin import PluginXMLFormat

current_path = os.path.abspath(os.getcwd())


class NmapXmlParser:
    """
    The objective of this class is to parse an xml file generated by
    the nmap tool.

    TODO: Handle errors.
    TODO: Test nmap output version. Handle what happens if the parser
    doesn't support it.
    TODO: Test cases.

    @param nmap_xml_filepath A proper xml generated by nmap
    """

    def __init__(self, xml_output):
        tree = self.parse_xml(xml_output)

        if tree:
            self.hosts = [host for host in self.get_hosts(tree)]
        else:
            self.hosts = []

    def parse_xml(self, xml_output):
        """
        Open and parse an xml file.

        TODO: Write custom parser to just read the nodes that we need instead
         of reading the whole file.

        @return xml_tree An xml tree instance. None if error.
        """

        try:
            magical_parser = XMLParser(recover=True)
            return etree.parse(BytesIO(xml_output), magical_parser)
        except SyntaxError as err:
            # logger.error("SyntaxError: %s." % (err))
            return None

    def get_hosts(self, tree):
        """
        @return hosts A list of Host instances
        """
        for host_node in tree.findall('host'):
            yield Host(host_node)


def get_attrib_from_subnode(xml_node, subnode_xpath_expr, attrib_name):
    """
    Finds a subnode in the host node and the retrieves a value from it

    @return An attribute value
    """

    node = xml_node.find(subnode_xpath_expr)

    if node is not None:
        return node.get(attrib_name)

    return None


class Host:
    """
    An abstract representation of a Host

    TODO: Consider evaluating the attributes lazily
    TODO: Write what's expected to be present in the nodes
    TODO: Refactor both Host and the Port clases?

    @param host_node A host_node taken from an nmap xml tree
    """

    def __init__(self, host_node):
        self.node = host_node

        self.hostnames = [hostname[0] for hostname in self.get_hostnames()]
        if len(self.hostnames) != 0:
            self.hostname = self.hostnames[0]
        else:
            self.hostname = 'unknown'

        self.hostnames = list(set(self.hostnames))
        self.status = self.get_status()
        self.ipv4_address = self.get_ipv4_address()
        self.ipv6_address = self.get_ipv6_address()
        self.mac_address = self.get_mac_address()
        self.os_guesses = [os_guess for os_guess in self.get_os_guesses()]
        self.os = self.top_os_guess()
        self.ports = [port for port in self.get_ports()]
        self.vulns = [vuln for vuln in self.get_scripts()]
        if self.os != 'unknown':
            for p in self.ports:
                if p.service is not None:
                    if p.service.ostype:
                        self.os = p.service.ostype
                        break

    def get_hostnames(self):
        """
        Expects to find one or more
        '<hostname name="localhost.localdomain" type="PTR"/>' in the host node.

        @return A list of (hostname, hostname_type) or None
        """
        for hostname in self.node.findall('hostnames/hostname'):
            yield (hostname.attrib["name"], hostname.attrib["type"])

    def get_attrib_from_subnode(self, subnode_xpath_expr, attrib_name):
        """
        Finds a subnode in the host node and the retrieves a value from it

        @return An attribute value
        """
        return get_attrib_from_subnode(
            self.node,
            subnode_xpath_expr,
            attrib_name)

    def get_status(self):
        """
        Expects to find '<status state="up" reason="conn-refused"/>'
        in the node
        TODO: Use 'reason'
        @return An status or 'unknown'
        """
        status = self.get_attrib_from_subnode('status', 'state')

        return status if status else 'unknown'

    def get_ipv4_address(self):
        """
        Expects to find '<address addr="127.0.0.1" addrtype="ipv4"/>'
        in the node

        @return ip_address or 'unknown'
        """
        ip_address = self.get_attrib_from_subnode(
            "address[@addrtype='ipv4']",
            'addr')
        return ip_address if ip_address else 'unknown'

    def get_ipv6_address(self):
        """
        Expects to find '<address addr="127.0.0.1" addrtype="ipv6"/>'
        in the node

        @return ip_address or 'unknown'
        """
        ip_address = self.get_attrib_from_subnode(
            "address[@addrtype='ipv6']",
            'addr')
        return ip_address if ip_address else 'unknown'

    def get_mac_address(self):
        """
        Expects to find
        '<address addr="00:08:54:26:A9:E5" addrtype="mac" vendor="Netronix" />'
        in the node

        @return mac_address or 'unknown'
        """
        mac_address = self.get_attrib_from_subnode(
            "address[@addrtype='mac']",
            'addr')
        return mac_address if mac_address else 'unknown'

    def get_os_guesses(self):
        """
        Expects to find
        '<os>..<osclass type="general purpose" vendor="Microsoft"
        osfamily="Windows" osgen="2003" accuracy="96" />..</os>' in the node

        @return A list of (os_vendor_family_gen, accuracy)
        """
        # OS information about host with great acurracy.

        osclasses = self.node.findall('os/osclass')
        if osclasses == []:
            osclasses = self.node.findall('os/osmatch/osclass')

        for osclass in osclasses:
            os_vendor = osclass.get("vendor", "unknown")
            os_family = osclass.get("osfamily", "unknown")
            os_gen = osclass.get("osgen", "unknown")
            accuracy = osclass.get("accuracy", "unknown")

            yield (f"{os_vendor} {os_family} {os_gen}", accuracy)

        # Os information in services, bad acurracy.
        if osclasses == []:
            services = self.node.findall("ports/port/service")
            for service in services:
                ostype = service.get("ostype", "unknown")
                yield (f"{ostype}", 0)

    def top_os_guess(self):
        """
        @return The most accurate os_guess_id or 'unknown'.
        """
        return self.os_guesses[0][0] if len(self.os_guesses) != 0 else 'unknown'

    def get_scripts(self):
        # Expects to find a scripts in the node.
        for s in self.node.findall('hostscript/script'):
            yield Script(s)

    def get_ports(self):
        """
        Expects to find one or more
        '<port protocol="tcp" portid="631">...</port>' in the node.

        @return A list of Port instances or None
        """
        for port in self.node.findall('ports/port'):
            yield Port(port)

    def is_up(self):
        """
        Returns True if the host is up else False.
        """
        if self.status == 'up':
            return True
        else:
            return False

    def __str__(self):
        ports = []
        for port in self.ports:
            var = f"    {port}"
            ports.append(var)
        ports = "\n".join(ports)

        return f"{self.hostnames}, {self.status}, {self.ipv4_address} [{self.mac_address}], {self.os}\n{ports}"


class Port:
    """
    An abstract representation of a Port.

    @param port_node A port_node taken from an nmap xml tree
    """

    PORT_STATUS_FIX = {"filtered": "closed", "open|filtered": "closed"}

    def __init__(self, port_node):
        self.node = port_node

        self.protocol = self.node.get("protocol")
        self.number = self.node.get("portid")
        self.state, self.reason, self.reason_ttl = self.get_state()
        self.service = self.get_service()
        self.vulns = [vuln for vuln in self.get_scripts()]

    def get_attrib_from_subnode(self, subnode_xpath_expr, attrib_name):
        """
        Finds a subnode in the host node and the retrieves a value from it.

        @return An attribute value
        """
        return get_attrib_from_subnode(
            self.node,
            subnode_xpath_expr,
            attrib_name)

    def get_state(self):
        """
        Expects to find a
        '<state state="open" reason="syn-ack" reason_ttl="0"/>' in the node.

        @return (state, reason, reason_ttl) or ('unknown','unknown','unknown')
        """
        state = self.PORT_STATUS_FIX.get(self.get_attrib_from_subnode('state', 'state'),
                                         self.get_attrib_from_subnode('state', 'state'))
        reason = self.get_attrib_from_subnode('state', 'reason')
        reason_ttl = self.get_attrib_from_subnode('state', 'reason_ttl')

        return (state if state else 'unknown',
                reason if reason else 'unknown',
                reason_ttl if reason_ttl else 'unknown')

    def get_service(self):
        """
        Expects to find a service in the node.
        """
        service_node = self.node.find('service')
        if service_node is not None:
            return Service(service_node)

        return None

    def get_scripts(self):
        """
        Expects to find a scripts in the node.
        """
        for s in self.node.findall('script'):
            if s.get("id") == 'vulners':
                for service_table in s.findall('table'):
                    for vulnerability_table in service_table.findall('table'):
                        yield ScriptVulners(s, service_table, vulnerability_table)
            else:
                yield Script(s)

    def __str__(self):
        return f"{self.number}, {self.state}, Service: {self.service}"


class ScriptVulners:
    """
    An abstract representation of a script table when script is 'vulners'.
    https://nmap.org/nsedoc/scripts/vulners.html

    '<script id="vulners" output="&#xa;  cpe:/a:apache:http_server:2.4.6: ...">
    <table key="cpe:/a:apache:http_server:2.4.6">
    <table><elem key="cvss">7.5</elem>
    <elem key="id">CVE-2017-7679</elem><elem key="type">cve</elem>
    <elem key="is_exploit">false</elem></table>
    ...
    </table></script>'

    @param script_node The reference to the node of the 'vulners' script
    @param service_table The outer table taken from an nmap 'vulners' script xml tree
    @param vulnerability_table The inner table taken from an nmap 'vulners' script xml tree
    """

    def __init__(self, script_node, service_table, vulnerability_table):
        self.node = script_node
        self.table = {}
        for e in vulnerability_table.findall('elem'):
            self.table[e.get("key")] = str(e.text)

        self.name = self.table["id"]

        self.desc = script_node.get("id") + "-" + self.table["id"]
        if self.table["is_exploit"] == 'true':
            self.desc += " *EXPLOIT*"

        self.refs = ["https://vulners.com/" + self.table["type"] + "/" + self.table["id"]]
        self.response = ""
        self.web = ""
        self.cvss2 = {}

    def __str__(self):
        return f"{self.name}, {self.product}, {self.version}"


class Script:
    """
    An abstract representation of a Script.

    '<script id="http-methods" output="No Allow or Public header in OPTIONS
    response (status code 400)"/><script id="http-title"
    output="Document Error: Unauthorized"><elem key="title">
    Document Error: Unauthorized</elem></script>'

    @param script_node A script_node taken from an nmap xml tree
    """

    def parse_output(self, output):
        block_re = re.compile('^\\s{4}References:((?:.|[\r\n])+[\r\n](?:\\s{4}\\w|\\s*$))', re.MULTILINE)
        m1 = block_re.findall(output)
        if len(m1) > 0:
            links_re = re.compile('[ \t]+([^ \t\n\r]+)[ \t]*')
            m2 = links_re.findall(m1[0])
            return m2
        return []

    def __init__(self, script_node):
        self.node = script_node

        self.name = script_node.get("id")
        self.desc = script_node.get("output")
        self.refs = self.parse_output(self.desc)
        self.response = ""
        for k in script_node.findall("elem"):
            self.response += "\n" + str(k.get('key')) + ": " + str(k.text)
        self.web = re.search("(http-|https-)", self.name)
        self.cvss2 = {}

    def __str__(self):
        return f"{self.name}, {self.product}, {self.version}"


class Service:
    """
    An abstract representation of a Service.

    '<service name="ipp" product="CUPS" version="1.4" method="probed"
     conf="10"/>'

    @param service_node A service_node taken from an nmap xml tree
    """

    def __init__(self, service_node):
        self.node = service_node

        name = service_node.get("name")
        self.name = name if name else 'unknown'

        self.tunnel = service_node.get("tunnel")
        if self.tunnel == "ssl":
            if self.name == "http":
                self.name = "https"
            elif self.name == 'imap':
                self.name = 'imaps'
            elif self.name == 'pop3':
                self.name = 'pop3s'

        product = service_node.get("product")
        self.product = product if product else 'unknown'

        version = service_node.get("version")
        self.version = version if version else 'unknown'

        self.method = service_node.get("method")
        self.conf = service_node.get("conf")
        self.ostype = self.node.get("ostype")

    def __str__(self):
        return f"{self.name}, {self.product}, {self.version}"


class NmapPlugin(PluginXMLFormat):
    """
    Example plugin to parse nmap output.
    """

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.identifier_tag = "nmaprun"
        self.id = "Nmap"
        self.name = "Nmap XML Output Plugin"
        self.plugin_version = "0.0.3"
        self.version = "6.40"
        self.framework_version = "1.0.0"
        self.options = None
        self._current_output = None
        self._command_regex = re.compile(r'^(sudo nmap|nmap|\.\/nmap|sudo masscan|masscan)\s+.*?')
        self._use_temp_file = True
        self._temp_file_extension = "xml"
        self.xml_arg_re = re.compile(r"^.*(-oX\s*[^\s]+).*$")
        self.addSetting("Scan Technique", str, "-sS")

    def parseOutputString(self, output):
        """
        This method will discard the output the shell sends, it will read it
        from the xml where it expects it to be present.

        NOTE: if 'debug' is true then it is being run from a test case and the
        output being sent is valid.
        """

        parser = NmapXmlParser(output)

        for host in parser.hosts:
            # if not host.is_up():
            #     continue

            if host.mac_address == 'unknown':
                host.mac_address = "00:00:00:00:00:00"

            if host.ipv4_address != 'unknown':
                minterfase = host.ipv4_address
            else:
                minterfase = host.ipv6_address
            h_id = self.createAndAddHost(minterfase, host.os, mac=host.mac_address, hostnames=host.hostnames)

            for v in host.vulns:
                desc = v.desc
                desc += "\nOutput: " + v.response if v.response else ""

                v_id = self.createAndAddVulnToHost(
                    h_id,
                    v.name,
                    desc=v.desc,
                    ref=v.refs,
                    severity=0,
                    cve=[v.name]
                )

            for port in host.ports:

                srvname = str(port.number)
                srvversion = "unknown"
                if port.service is not None:
                    srvname = port.service.name
                    srvversion = port.service.product if port.service.product != "unknown" else ""
                    srvversion += " " + port.service.version if port.service.version != "unknown" else ""

                s_id = self.createAndAddServiceToHost(
                    h_id,
                    srvname,
                    port.protocol,
                    ports=[port.number],
                    status=port.state,
                    version=srvversion,
                    description=srvname)

                for v in port.vulns:

                    desc = v.desc
                    refs = v.refs

                    if hasattr(v, 'severity'):
                        severity = v.severity
                    else:
                        severity = "info"
                        if re.search(r"(?<!NOT )VULNERABLE", desc):
                            severity = "high"
                        if re.search(r"ERROR", desc):
                            severity = "unclassified"
                        if re.search(r"Couldn't", desc):
                            severity = "unclassified"

                    if v.web:
                        v_id = self.createAndAddVulnWebToService(
                            h_id,
                            s_id,
                            v.name,
                            desc=desc,
                            response=v.response if v.response else "",
                            ref=refs,
                            severity=severity,
                            website=minterfase,
                            cve=[v.name],
                            cvss2=v.cvss2
                        )
                    else:
                        v_id = self.createAndAddVulnToService(
                            h_id,
                            s_id,
                            v.name,
                            desc=v.desc,
                            ref=refs,
                            severity=severity,
                            cve=[v.name],
                            cvss2=v.cvss2
                        )
        del parser

    def processCommandString(self, username, current_path, command_string):
        """
        Adds the -oX parameter to get xml output to the command string that the
        user has set.
        """
        super().processCommandString(username, current_path, command_string)
        arg_match = self.xml_arg_re.match(command_string)
        if arg_match is None:
            if "masscan" in command_string:
                return re.sub(r"(^.*?masscan)",
                              r"\1 -oX %s" % self._output_file_path,
                              command_string)
            else:
                return re.sub(r"(^.*?nmap)",
                              r"\1 -oX %s" % self._output_file_path,
                              command_string)
        else:
            return re.sub(arg_match.group(1),
                          r"-oX %s" % self._output_file_path,
                          command_string)


def createPlugin(*args, **kwargs):
    return NmapPlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2013  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information

"""
import re
import xml.etree.ElementTree as ET
from urllib.parse import urlparse

from faraday_plugins.plugins.plugin import PluginXMLFormat

__author__ = "Francisco Amato"
__copyright__ = "Copyright (c) 2013, Infobyte LLC"
__credits__ = ["Francisco Amato"]
__license__ = ""
__version__ = "1.0.0"
__maintainer__ = "Francisco Amato"
__email__ = "famato@infobytesec.com"
__status__ = "Development"


class W3afXmlParser:
    """
    The objective of this class is to parse an xml file generated by the w3af tool.

    TODO: Handle errors.
    TODO: Test w3af output version. Handle what happens if the parser doesn't support it.
    TODO: Test cases.

    @param w3af_xml_filepath A proper xml generated by w3af
    """

    def __init__(self, xml_output):
        self.target = None
        self.port = "80"
        self.host = None

        tree = self.parse_xml(xml_output)

        if tree:
            self.items = [data for data in self.get_items(tree)]
        else:
            self.items = []

    def parse_xml(self, xml_output):
        """
        Open and parse an xml file.

        TODO: Write custom parser to just read the nodes that we need instead of
        reading the whole file.

        @return xml_tree An xml tree instance. None if error.
        """
        try:
            tree = ET.fromstring(xml_output)
        except SyntaxError as err:
            return None

        return tree

    def get_items(self, tree):
        """
        @return items A list of Host instances
        """

        if len(tree.findall('scan-info')) == 0:
            scaninfo = tree.findall('scaninfo')[0]
        else:
            scaninfo = tree.findall('scan-info')[0]

        self.target = scaninfo.get('target')
        url_parse = urlparse(self.target)

        self.protocol = url_parse.scheme
        self.host = url_parse.netloc
        self.port = url_parse.port
        if self.port is None:
            if self.protocol == 'https':
                self.port = 443
            elif self.protocol == 'http':
                self.port = 80

        for node in tree.findall('vulnerability'):
            yield Item(node)
        for node in tree.findall('information'):
            yield Item(node)


def get_attrib_from_subnode(xml_node, subnode_xpath_expr, attrib_name):
    """
    Finds a subnode in the item node and the retrieves a value from it

    @return An attribute value
    """
    node = xml_node.find(subnode_xpath_expr)

    if node is not None:
        return node.get(attrib_name)

    return None


class Item:
    """
    An abstract representation of a Item


    @param item_node A item_node taken from an w3af xml tree
    """

    def __init__(self, item_node):
        self.node = item_node

        self.id = self.node.get('id')
        self.name = self.node.get('name')
        self.url = self.node.get('url')
        self.url = self.url if self.url != 'None' else "/"
        self.plugin = self.node.get('plugin')
        self.detail = self.get_text_from_subnode('description')
        if not self.detail:
            self.detail = self.node.text.strip('\n').strip()
        self.resolution = self.get_text_from_subnode('fix-guidance')
        self.fix_effort = self.get_text_from_subnode('fix-effort')
        self.longdetail = self.get_text_from_subnode('long-description')
        self.severity = self.node.get('severity')
        self.method = self.node.get('method')
        self.ref = []
        self.param = self.node.get('var') if self.node.get(
            'var') != "None" else ""
        for ref in self.node.findall('references/reference'):
            self.ref.append(ref.get('url'))

        self.req = self.resp = ''
        for tx in self.node.findall('http-transactions/http-transaction'):

            if tx.find('http-request'):
                hreq = tx.find('http-request')
            else:
                hreq = tx.find('httprequest')

            if tx.find('http-response'):
                hres = tx.find('http-response')
            else:
                hres = tx.find('httpresponse')

            self.req = hreq.find('status').text
            for h in hreq.findall('headers/header'):
                self.req += f"\n{h.get('field')}: {h.get('content')}"

            if hreq.findtext('body', ""):
                self.req += f"\n{hreq.findtext('body', '')}"

            self.resp = hres.find('status').text
            for h in hres.findall('headers/header'):
                self.resp += f"\n{h.get('field')}: {h.get('content')}"

            if hres.findtext('body', ""):
                self.resp += f"\n{hres.findtext('body', '')}"

    def do_clean(self, value):
        myreturn = ""
        if value is not None:
            myreturn = re.sub("\n", "", value)
        return myreturn

    def get_text_from_subnode(self, subnode_xpath_expr):
        """
        Finds a subnode in the host node and the retrieves a value from it.

        @return An attribute value
        """
        sub_node = self.node.find(subnode_xpath_expr)
        if sub_node is not None:
            return sub_node.text

        return None


class W3afPlugin(PluginXMLFormat):
    """
    Example plugin to parse w3af output.
    """

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.identifier_tag = ["w3af-run", "w3afrun"]
        self.id = "W3af"
        self.name = "W3af XML Output Plugin"
        self.plugin_version = "0.0.2"
        self.version = "1.7.6"
        self.framework_version = "1.0.0"
        self.options = None
        self._current_output = None
        self.target = None
        self._command_regex = re.compile(r'^(w3af|sudo w3af|\.\/w3af)\s+.*?')
        self._completition = {
            "": "",
            "-h": "Display this help message.",
        }

    def parseOutputString(self, output):
        parser = W3afXmlParser(output)
        ip = self.resolve_hostname(parser.host)
        h_id = self.createAndAddHost(ip, hostnames=[parser.host])
        s_id = self.createAndAddServiceToHost(h_id, "http", "tcp", ports=[parser.port], status="open")

        for item in parser.items:
            self.createAndAddVulnWebToService(h_id, s_id, item.name,
                                                     item.detail, pname=item.param,data=item.longdetail, path=item.url, website=parser.host,
                                                     severity=item.severity, method=item.method, request=item.req,
                                                     resolution=item.resolution, ref=item.ref, response=item.resp)
        del parser


def createPlugin(*args, **kwargs):
    return W3afPlugin(*args, **kwargs)



class Base:
    def __init__(self, node):
        self.node = node


class Cvss(Base):
    @property
    def v2vector(self):
        return self.node.get("V2Vector")

    @property
    def v3vector(self):
        return self.node.get("V3Vector")

    @property
    def v2score(self):
        return self.node.get("V2Score")

    @property
    def v3score(self):
        return self.node.get("V3Score")


class Line(Base):

    @property
    def number(self):
        return self.node.get("Number")

    @property
    def content(self):
        return self.node.get("Content")


class Code(Base):

    @property
    def lines(self):
        if self.node.get("Lines"):
            return [Line(i) for i in self.node.get("Lines")]
        else:
            return None


class CauseMetadata(Base):

    @property
    def code(self):
        return Code(self.node.get("Code"))


class Misconfiguration(Base):

    @property
    def misconfig_type(self):
        return self.node.get("Type")

    @property
    def misconfig_id(self):
        return self.node.get("ID")

    @property
    def title(self):
        return self.node.get("Title")

    @property
    def description(self):
        description = self.node.get("Description")
        if description:
            return description
        else:
            return "Issues provided no description"

    @property
    def message(self):
        return self.node.get("Message")

    @property
    def resolution(self):
        return self.node.get("Resolution")

    @property
    def severity(self):
        return self.node.get("Severity")

    @property
    def references(self):
        return self.node.get("References")

    @property
    def cause_metadata(self):
        return CauseMetadata(self.node.get("CauseMetadata"))


class Vulnerability(Base):

    @property
    def name(self):
        return self.node.get("VulnerabilityID")

    @property
    def title(self):
        return self.node.get("Title")

    @property
    def pkgname(self):
        return self.node.get("PkgName")

    @property
    def url(self):
        return self.node.get("PrimaryURL")

    @property
    def description(self):
        description = self.node.get("Description")
        if description:
            return description
        else:
            return "Issues provided no description"

    @property
    def severity(self):
        return self.node.get("Severity")

    @property
    def cwe(self):
        return self.node.get("CweIDs")

    @property
    def cvss(self):
        if self.node.get("cvss"):
            return Cvss(self.node.get("cvss").get("nvd"))
        else:
            return None

    @property
    def references(self):
        return self.node.get("References")


class Result(Base):

    @property
    def target(self):
        return self.node.get("Target")

    @property
    def misconfigurations(self):
        return [Misconfiguration(i) for i in self.node.get("Misconfigurations", [])]

    @property
    def vulnerability(self):
        return [Vulnerability(i) for i in self.node.get("Vulnerabilities", [])]

    @property
    def result_type(self):
        return self.node.get("Type")


class Metadata(Base):

    @property
    def os_family(self):
        return self.node.get("Family")

    @property
    def os_name(self):
        return self.node.get("Name")


class TrivyJsonParser(Base):

    @property
    def results(self):
        return [Result(i) for i in self.node.get('Results', "")]

    @property
    def scantype(self):
        return self.node.get('ArtifactType')

    def metadata(self):
        return Metadata(self.node.get('Metadata'))



"""
Faraday Penetration Test IDE
Copyright (C) 2013  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information

"""
import re
import json
import argparse
import shlex
import gzip
import os
import shutil

from faraday_plugins.plugins.plugin import PluginMultiLineJsonFormat
from faraday_plugins.plugins.plugins_utils import get_severity_from_cvss

__author__ = "Valentin Vila"
__copyright__ = "Copyright (c) 2021, Faraday"
__credits__ = ["Valentin Vila"]
__license__ = ""
__version__ = "1.0.0"
__maintainer__ = "Valentin Vila"
__email__ = "vvila@faradaysec.com"
__status__ = "Development"


class ShodanPlugin(PluginMultiLineJsonFormat):
    """
    This plugin handles the Shodan tool.
    Detects the output of the tool
    and adds the information to Faraday.
    """

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.id = "shodan"
        self.name = "Shodan"
        self.plugin_version = "0.0.1"
        self.version = "1.0.0"
        self._command_regex = re.compile(r'^shodan\s+(?P<option>search|download)\s+.*\w+')
        self._use_temp_file = True
        self._temp_file_extension = "json.gz"
        self.json_keys = {'_shodan'}

    def _parse_filename(self, filename):
        if self.has_custom_output():
            with gzip.open(filename, 'rb') as output:
                self.parseOutputString(output.read().decode('utf-8'))
        else:
            with open(filename) as output:
                self.parseOutputString(output.read())
        if self._delete_temp_file:
            try:
                if os.path.isfile(filename):
                    os.remove(filename)
                elif os.path.isdir(filename):
                    shutil.rmtree(filename)
            except Exception as e:
                self.logger.error(f"Error on delete file: ({filename}) [{e}]")

    def parseOutputString(self, output):
        for vuln_json in filter(lambda x: x != '', output.split("\n")):
            vuln_dict = json.loads(vuln_json)
            ip = vuln_dict.get('ip_str')
            port = vuln_dict.get('port')
            vulns = vuln_dict.get('vulns')
            transport = vuln_dict.get('transport')
            hostnames = vuln_dict.get('hostnames')
            h_id = self.createAndAddHost(ip, hostnames=hostnames)
            s_id = self.createAndAddServiceToHost(h_id, "http", protocol=transport, ports=port)
            if vulns is not None:
                for name, vuln_info in vulns.items():
                    description = vuln_info.get('summary')
                    references = vuln_info.get('references')
                    self.createAndAddVulnToService(h_id, s_id, name, desc=description, ref=references
                                                   , cve=name)

    def processCommandString(self, username, current_path, command_string):
        """
        Adds the path to a temporary file parameter to get .json.gz output to the command string that the
        user has set.
        """
        super().processCommandString(username, current_path, command_string)
        parser = argparse.ArgumentParser(conflict_handler='resolve')
        match = self._command_regex.match(command_string)
        if match.groupdict()['option'] == 'search':
            parser.add_argument("--color", action="store_true")
            parser.add_argument("--no-color", action="store_true")
            parser.add_argument("--fields", type=str)
            parser.add_argument("--limit", type=int)
            parser.add_argument("--separator", type=str, nargs=-1)
            parser.add_argument("query")
        else:
            parser.add_argument("--limit", type=int)
            parser.add_argument("query", type=str, nargs=-1)
        args = parser.parse_args(shlex.split(command_string.split("search ")[-1]))
        limit = args.limit
        query = args.query
        cmd = "shodan download"
        if limit:
            cmd += f" --limit {limit}"
        cmd += f" {self._output_file_path} {query}"
        return cmd


def createPlugin(*args, **kwargs):
    return ShodanPlugin(*args, **kwargs)



import re
from faraday_plugins.plugins.plugin import PluginXMLFormat

try:
    from lxml import etree as ET
except ImportError:
    import xml.etree.ElementTree as ET

WEAK_CIPHER_LIST = [
    "TLS_ECDHE_RSA_WITH_3DES_EDE_CBC_SHA",
    "TLS_RSA_WITH_AES_128_CBC_SHA",
    "TLS_RSA_WITH_AES_128_CBC_SHA256",
    "TLS_RSA_WITH_AES_128_GCM_SHA256",
    "TLS_RSA_WITH_AES_256_CBC_SHA",
    "TLS_RSA_WITH_AES_256_CBC_SHA256",
    "TLS_RSA_WITH_AES_256_GCM_SHA384",
    "TLS_RSA_WITH_3DES_EDE_CBC_SHA",
    "TLS_RSA_WITH_CAMELLIA_256_CBC_SHA",
    "TLS_RSA_WITH_CAMELLIA_128_CBC_SHA"
]


class SslyzeXmlParser:

    def __init__(self, xml_output):
        self.parser = self.parse_xml(xml_output)
        self.target = self.get_target(self.parser)
        self.certificate = self.get_hostname_validation(self.parser)
        self.cipher_suite = self.get_weak_cipher_suite(self.parser)
        self.heart_bleed = self.get_heartbleed(self.parser)
        self.open_ssl_ccs = self.get_openssl_ccs(self.parser)

    def parse_xml(self, xml_output):
        try:
            tree = ET.fromstring(xml_output)
            return tree
        except IndexError:
            print("Syntax error")
            return None

    def get_target(self, tree):
        return tree.xpath('//target')

    def get_hostname_validation(self, tree):
        return tree.xpath('//hostnameValidation')

    def get_protocol_name(self, tree):
        protocol_supported = []
        protocols = []
        protocols.append(tree.xpath('//sslv2'))
        protocols.append(tree.xpath('//sslv3'))
        protocols.append(tree.xpath('//tlsv1'))
        protocols.append(tree.xpath('//tlsv1_1'))
        protocols.append(tree.xpath('//tlsv1_2'))
        protocols.append(tree.xpath('//tlsv1_3'))

        for protocol in protocols:
            if protocol[0].attrib['isProtocolSupported'] == "True":
                protocol_supported.append(protocol[0])

        return protocol_supported

    def get_weak_cipher_suite(self, tree):
        protocols = self.get_protocol_name(tree)
        weak_cipher = {}

        for protocol in protocols:
            weak_cipher[protocol.tag] = []
            for ciphers in protocol:
                if ciphers.tag == 'preferredCipherSuite' or ciphers.tag == 'acceptedCipherSuites':
                    for cipher in ciphers:
                        if cipher.attrib['name'] in WEAK_CIPHER_LIST:
                            if not cipher.attrib['name'] in weak_cipher[protocol.tag]:
                                weak_cipher[protocol.tag].append(cipher.attrib['name'])

        return weak_cipher

    def get_heartbleed(self, tree):
        return tree.xpath('//heartbleed')

    def get_openssl_ccs(self, tree):
        return tree.xpath('//openssl_ccs')


class SslyzePlugin(PluginXMLFormat):

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.identifier_tag = "document"
        self.identifier_tag_attributes = {'SSLyzeVersion'}
        self.id = "Sslyze_XML"
        self.name = "Sslyze Plugin"
        self.plugin_version = "0.0.1"
        self.version = "2.0.6"
        self.framework_version = "1.0.0"
        self.options = None
        self._current_output = None
        self._use_temp_file = True
        self._temp_file_extension = "xml"

    def report_belongs_to(self, **kwargs):
        if super().report_belongs_to(**kwargs):
            report_path = kwargs.get("report_path", "")
            with open(report_path) as f:
                output = f.read()
            return re.search("SSLyzeVersion", output) is not None
        return False

    def parseOutputString(self, output):
        parser = SslyzeXmlParser(output)
        host = parser.target[0].attrib['host']
        ip = parser.target[0].attrib['ip']
        port = parser.target[0].attrib['port']
        protocol = parser.target[0].attrib['tlsWrappedProtocol']
        cipher = parser.cipher_suite

        # Creating host
        host_id = self.createAndAddHost(ip)
        # Creating service CHANGE NAME
        service_id = self.createAndAddServiceToHost(
            host_id,
            name=protocol,
            protocol=protocol,
            ports=[port],
        )

        # Checking if certificate matches
        certificate = parser.certificate[0].attrib['certificateMatchesServerHostname']
        server_hostname = parser.certificate[0].attrib['serverHostname']
        if certificate.lower() == 'false':
            self.createAndAddVulnToService(
                host_id,
                service_id,
                name="Certificate mismatch",
                desc=f"Certificate does not match server hostname {server_hostname}",
                severity="info")
        # Ciphers
        cipher = parser.cipher_suite

        for key in cipher:
            for value in cipher[key]:
                self.createAndAddVulnToService(
                    host_id,
                    service_id,
                    name=value,
                    desc=f"In protocol [{key}], weak cipher suite: {value}",
                    severity="low")

        # Heartbleed
        heartbleed = parser.heart_bleed

        if heartbleed[0][0].attrib['isVulnerable'].lower() == 'true':
            self.createAndAddVulnToService(
                host_id,
                service_id,
                name="OpenSSL Heartbleed",
                desc="OpenSSL Heartbleed is vulnerable",
                severity="critical")

        # OpenSsl CCS Injection
        openssl_ccs = parser.open_ssl_ccs

        if openssl_ccs[0][0].attrib['isVulnerable'].lower() == 'true':
            self.createAndAddVulnToService(
                host_id,
                service_id,
                name="OpenSSL CCS Injection",
                desc="OpenSSL CCS Injection is vulnerable",
                severity="medium")


def createPlugin(*args, **kwargs):
    return SslyzePlugin(*args, **kwargs)



from urllib.parse import urlparse

from faraday_plugins.plugins.plugin import PluginXMLFormat

import xml.etree.ElementTree as ET

__author__ = "Nicolas Rebagliati"
__copyright__ = "Copyright (c) 2021, Infobyte LLC"
__credits__ = ["Nicolas Rebagliati"]
__license__ = ""
__version__ = "1.0"
__maintainer__ = "Nicolas Rebagliati"
__status__ = "Development"


class AppScanParser:

    def __init__(self, xml_output):
        tree = self.parse_xml(xml_output)
        if tree:
            self.scan_type = tree.attrib['technology']
            self.issue_types = self.get_issue_types(tree.find('issue-type-group'))
            if self.scan_type == "SAST":
                self.fixes = self.get_fixes(tree.find("fix-group-group"))
                self.issues = self.get_sast_issues(tree.find("issue-group"))
            elif self.scan_type == "DAST":
                self.hosts = self.get_hosts(tree.find('scan-configuration/scanned-hosts'))
                self.remediations = self.get_remediations(tree.find('remediation-group'))
                self.entities = self.get_entity_groups(tree.find('entity-group'))
                self.issues = self.get_dast_issues(tree.find("issue-group"))

    @staticmethod
    def parse_xml(xml_output):
        try:
            tree = ET.fromstring(xml_output)
        except SyntaxError as err:
            print(f'SyntaxError In xml: {err}. {xml_output}')
            return None
        return tree

    @staticmethod
    def get_fixes(tree):
        fixes = {}
        for item in tree:
            fix_id = item.attrib['id']
            library = item.find("LibraryName").text
            location = item.find("Location").text
            fixes[fix_id] = {"library": library, "location": location}
        return fixes

    @staticmethod
    def get_issue_types(tree):
        issue_types = {}
        for item in tree:
            type_id = item.attrib['id']
            name = item.find("name").text
            issue_types[type_id] = name
            cve = item.find("cve")
            if cve and cve.text:
                issue_types[f"{type_id}_cve"] = cve.text
        return issue_types

    @staticmethod
    def get_remediations(tree):
        remediations = {}
        for item in tree:
            remediation_id = item.attrib['id']
            name = item.find("name").text
            remediations[remediation_id] = name
        return remediations

    @staticmethod
    def get_hosts(tree):
        hosts = {}
        for item in tree:
            host = item.find("host").text
            port = item.find("port").text
            operating_system = item.find("operating-system").text
            if "unknown" in operating_system.lower():
                operating_system = "unknown"
            web_server = item.find("web-server").text
            application_server = item.find("application-server").text
            service_name = f"{web_server} ({application_server})"
            host_key = f"{host}-{port}"
            hosts[host_key] = {"host": host, "port": port, "os": operating_system,
                               "service_name": service_name}
        return hosts

    @staticmethod
    def get_entity_groups(tree):
        entity_groups = {}
        for item in tree:
            entity_id = item.attrib['id']
            name = item.find("name").text
            url = item.find("url-name").text
            type = item.find("entity-type").text
            url_data = urlparse(url)
            website = f"{url_data.scheme}://{url_data.netloc}"
            host = url_data.netloc.split(":")[0]
            if url_data.port:
                port = url_data.port
            else:
                if url_data.scheme == "http":
                    port = 80
                elif url_data.scheme == "https":
                    port = 443
            path = url_data.path
            entity_groups[entity_id] = {"name": name, "host": host, "port": port, "url": url,
                                        "type": type, "website": website, "path": path}
        return entity_groups

    def get_dast_issues(self, tree):
        dast_issues = []
        for item in tree:
            entity = self.entities[item.find("entity/ref").text]
            host = entity["host"].replace('\\','/')
            port = entity["port"]
            name = self.issue_types[item.find("issue-type/ref").text]
            severity = 0 if item.find("severity-id") is None else int(item.find("severity-id").text)
            resolution = self.remediations[item.find("remediation/ref").text]
            description = "" if item.find("variant-group/item/reasoning") is None \
                else item.find("variant-group/item/reasoning").text
            request = "" if item.find("variant-group/item/test-http-traffic") is None \
                else item.find("variant-group/item/test-http-traffic").text
            response = "" if item.find("variant-group/item/issue-information/testResponseChunk") is None \
                else item.find("variant-group/item/issue-information/testResponseChunk").text
            cvss2 = item.find('cvss-score').text if item.find("cvss-score") is not None else None
            cvss2_base_vector = item.find('cvss-vector/base-vector').text if item.find('cvss-vector/base-vector') \
                                                                             is not None else None
            cvss_temporal_vector = None if item.find('cvss-vector/temporal-vector') is None \
                else f"CVSS-temporal-vector: {item.find('cvss-vector/temporal-vector').text}"
            cvss_environmental_vector = None if item.find('cvss-vector/environmental-vector') is None \
                else f"CVSS-environmental-vector: {item.find('cvss-vector/environmental-vector').text}"
            cwe = None if item.find("cwe") is None else item.find('cwe').text
            if item.attrib.get("cve"):
                cve = None if item.find("variant-group/item/issue-information/display-name") is None \
                    else item.find('variant-group/item/issue-information/display-name').text
                if "CVE" not in cve:
                    cve = f"CVE-{cve}"
                cve_url = item.attrib["cve"]
            else:
                cve = None
                cve_url = None
            if cve is None:
                cve = self.issue_types.get(f"{item.find('issue-type/ref').text}_cve", None)
            host_key = f"{host}-{port}"
            issue_data = {
                "host": host,
                "port": port,
                "os": self.hosts[host_key]["os"],
                "service_name": self.hosts[host_key]["service_name"],
                "name": name,
                "severity": severity,
                "desc": description,
                "ref": [],
                "resolution": resolution,
                "request": request,
                "response": response,
                "website": entity['website'],
                "path": entity['path'],
                "cve": [],
                "cwe": [],
                "cvss2": {}
            }
            if cve:
                issue_data["cve"].append(cve)
                issue_data["desc"] += cve
            if cve_url:
                issue_data["ref"].append(cve_url)
            if cwe:
                issue_data["cwe"].append(f"CWE-{cwe}")
            if cvss2_base_vector:
                issue_data["cvss2"]["vector_string"] = cvss2_base_vector
            if cvss_temporal_vector:
                issue_data["ref"].append(cvss_temporal_vector)
            if cvss_environmental_vector:
                issue_data["ref"].append(cvss_environmental_vector)
            dast_issues.append(issue_data)
        return dast_issues

    def get_sast_issues(self, tree):
        sast_issues = []
        for item in tree:
            name = self.issue_types[item.find("issue-type/ref").text]
            source_file = item.attrib["filename"].replace('\\', '/')
            severity = 0 if item.find("severity-id") is None else int(item.find("severity-id").text)
            description = item.find("fix/item/general/text").text
            resolution = "" if item.find("variant-group/item/issue-information/fix-resolution-text") is None \
                else item.find("variant-group/item/issue-information/fix-resolution-text").text
            fix_id = item.attrib.get("fix-group-id")
            if fix_id:
                fix = self.fixes[fix_id]
                resolution = f"{resolution}\nLibrary: {fix['library']}\nLocation: {fix['location']}"
            cvss2 = item.find('cvss-score').text if item.find("cvss-score") else None
            cvss2_base_vector = None if item.find('cvss-vector/base-vector') is None \
                else item.find('cvss-vector/base-vector').text
            cvss_temporal_vector = None if item.find('cvss-vector/temporal-vector') is None \
                else f"CVSS-temporal-vector: {item.find('cvss-vector/temporal-vector').text}"
            cvss_environmental_vector = None if item.find('cvss-vector/environmental-vector') is None \
                else f"CVSS-environmental-vector: {item.find('cvss-vector/environmental-vector').text}"
            cwe = None if item.find("cwe/ref") is None else item.find('cwe/ref').text
            if item.attrib.get("cve"):
                cve = None if item.find("variant-group/item/issue-information/display-name") is None \
                    else item.find('variant-group/item/issue-information/display-name').text
                if "CVE" not in cve:
                    cve = f"CVE-{cve}"
                cve_url = item.attrib["cve"]
            else:
                cve = None
                cve_url = None
            issue_data = {
                "source_file": source_file,
                "name": name,
                "severity": severity,
                "desc": description,
                "ref": [],
                "resolution": resolution,
                "cve": [],
                "cwe": [],
                "cvss2": {}
            }

            if cve_url:
                issue_data["ref"].append(cve_url)
            if cwe:
                issue_data["cwe"].append(f"CWE-{cwe}")
            if cvss2_base_vector:
                issue_data["cvss2"]['vector_string'] = cvss2_base_vector
            if cvss_temporal_vector:
                issue_data["ref"].append(cvss_temporal_vector)
            if cvss_environmental_vector:
                issue_data["ref"].append(cvss_environmental_vector)
            if cve:
                issue_data["cve"].append(cve)
                issue_data["desc"] += f"\nCVE: {cve}"
            # Build data
            data = []
            if item.attrib.get("caller"):
                data.append(f"Caller: {item.attrib.get('caller')}")
            if item.find("variant-group/item/issue-information/method-signature") is not None:
                data.append(f"Method: {item.find('variant-group/item/issue-information/method-signature').text}")
            if item.find("variant-group/item/issue-information/method-signature2") is not None:
                data.append(f"Location: {item.find('variant-group/item/issue-information/method-signature2').text}")
            issue_data['data'] = "\n".join(data)
            issue_data['desc'] += "\n".join(data)
            sast_issues.append(issue_data)
        return sast_issues


class AppScanPlugin(PluginXMLFormat):

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.identifier_tag = "xml-report"
        self.id = 'Appscan'
        self.name = 'Appscan XML Plugin'
        self.plugin_version = '0.0.1'
        self.version = '1.0.0'
        self.framework_version = '1.0.0'

    def parseOutputString(self, output):
        parser = AppScanParser(output)
        scan_type = parser.scan_type

        if scan_type == 'DAST':
            for issue in parser.issues:
                host = issue.pop("host")
                port = issue.pop("port")
                service_name = issue.pop("service_name")
                ip = self.resolve_hostname(host)
                host_os = issue.pop("os")
                host_id = self.createAndAddHost(ip, hostnames=host, os=host_os)
                service_id = self.createAndAddServiceToHost(host_id, service_name, ports=port)
                self.createAndAddVulnWebToService(host_id=host_id, service_id=service_id, **issue)

        elif scan_type == 'SAST':
            for issue in parser.issues:
                source_file = issue.pop('source_file')
                host_id = self.createAndAddHost(source_file)
                self.createAndAddVulnToHost(host_id=host_id, **issue)


def createPlugin(*args, **kwargs):
    return AppScanPlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2013  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information
"""
from json import loads

from faraday_plugins.plugins.plugin import PluginJsonFormat
from faraday_plugins.plugins.repo.pentera.DTO import PenteraJsonParser

__author__ = "Gonzalo Martinez"
__copyright__ = "Copyright (c) 2013, Infobyte LLC"
__credits__ = ["Gonzalo Martinez"]
__license__ = ""
__version__ = "1.0.0"
__maintainer__ = "Gonzalo Martinez"
__email__ = "gmartinez@infobytesec.com"
__status__ = "Development"





class PenteraJsonPlugin(PluginJsonFormat):

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.id = "Pentera_Json"
        self.name = "Pentera Json Output Plugin"
        self.plugin_version = "1.0.0"
        self.version = "2.11.1"
        self.framework_version = "1.0.0"
        self.options = None
        self._temp_file_extension = "json"
        self.json_keys = {'vulnerabilities', 'hosts'}

    @staticmethod
    def pentera_to_severity_level(pentera_score):
        pentera_ranges = [(0.0, 0.01, 'info'),
                          (0.01, 2.5, 'low'),
                          (2.5, 4.5, 'med'),
                          (5, 7.49, 'high'),
                          (7.5, 10.1, 'critical')]
        for (lower, upper, severity) in pentera_ranges:
            if lower <= pentera_score < upper:
                return severity

    def parseOutputString(self, output):
        """
        This method will discard the output the shell sends, it will read it
        from the json where it expects it to be present.
        """

        parser = PenteraJsonParser(loads(output))
        host_dict = {}

        for host in parser.hosts:
            host_id = self.createAndAddHost(
                name=host.name,
                os=host.os,
                hostnames=[host.hostname]
            )
            host_dict[host.host_id] = {}
            for service in host.services:
                service_id = self.createAndAddServiceToHost(
                    host_id=host_id,
                    name=service.name,
                    protocol=service.protocol,
                    ports=service.port,
                    status=service.status
                )
                host_dict[host.host_id][service.port] = [host_id, service_id]
        for vuln in parser.vulneravilities:
            try:
                self.createAndAddVulnToService(
                    host_id=host_dict[vuln.host_id][vuln.port][0],
                    service_id=host_dict[vuln.host_id][vuln.port][1],
                    name=vuln.name,
                    desc=vuln.description,
                    severity=self.pentera_to_severity_level(vuln.severity),
                    resolution=vuln.resolution,
                    data=vuln.data,
                    external_id=vuln.external_id,
                    cve=vuln.cve
                )
            except KeyError:
                if "host" in vuln.found_on.lower():
                    host = vuln.found_on.lower().replace("host:", "").strip().split(",")[0]
                else:
                    host = "NOT-PROVIDED"
                host_id = self.createAndAddHost(
                    name=host,
                )
                self.createAndAddVulnToHost(
                    host_id=host_id,
                    name=vuln.name,
                    desc=vuln.description,
                    severity=self.pentera_to_severity_level(vuln.severity),
                    resolution=vuln.resolution,
                    data=vuln.data,
                    external_id=vuln.external_id,
                    cve=vuln.cve
                )


def createPlugin(*args, **kwargs):
    return PenteraJsonPlugin(*args, **kwargs)



from faraday_plugins.plugins.plugin import PluginXMLFormat
import xml.etree.ElementTree as ET
import re

class BanditPlugin(PluginXMLFormat):
    """
    Example plugin to parse bandit output.
    """

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.identifier_tag = 'testsuite'
        self.extension = ".xml"
        self.id = "Bandit"
        self.name = "Bandit XML Output Plugin"
        self.plugin_version = "0.0.1"

    def report_belongs_to(self, **kwargs):
        if super().report_belongs_to(**kwargs):
            report_path = kwargs.get("report_path", "")
            with open(report_path) as f:
                output = f.read()
            return re.search("testsuite name=\"bandit\"", output) is not None
        return False

    def parseOutputString(self, output):
        bp = BanditParser(output)

        for vuln in bp.vulns:
            host_id = self.createAndAddHost(vuln['path'])

            self.createAndAddVulnToHost(
                host_id=host_id,
                name=vuln['name'],
                desc=vuln['issue_text'],
                ref=vuln['references'],
                severity=vuln['severity'],
            )

        return True


class BanditParser:
    """
    Parser for bandit on demand
    """

    def __init__(self, xml_output):
        self.vulns = self._parse_xml(xml_output)


    def _parse_xml(self, xml_output):
        vulns = []
        tree = ET.fromstring(xml_output)
        testcases = tree.findall('testcase')

        for testcase in testcases:
            error = testcase.find('error')
            name = testcase.attrib['name']
            path = testcase.attrib['classname']
            severity = error.attrib['type']
            issue_text = error.text
            more_info = error.attrib['more_info']
            ref = [more_info]

            vulns.append({'name': name, 'path': path, 'references': ref, 'issue_text': issue_text, 'severity': severity})

        return vulns


def createPlugin(*args, **kwargs):
    return BanditPlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2013  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information

"""
from faraday_plugins.plugins.plugin import PluginBase
import re
import os



current_path = os.path.abspath(os.getcwd())

__author__ = "Francisco Amato"
__copyright__ = "Copyright (c) 2013, Infobyte LLC"
__credits__ = ["Francisco Amato"]
__license__ = ""
__version__ = "1.0.0"
__maintainer__ = "Francisco Amato"
__email__ = "famato@infobytesec.com"
__status__ = "Development"


class WebfuzzerParser:
    """
    The objective of this class is to parse an xml file generated by the webfuzzer tool.

    TODO: Handle errors.
    TODO: Test webfuzzer output version. Handle what happens if the parser doesn't support it.
    TODO: Test cases.

    @param webfuzzer_filepath A proper output generated by webfuzzer
    """

    def __init__(self, webfuzzer_filepath):
        self.filepath = webfuzzer_filepath

        with open(self.filepath) as f:
            try:
                data = f.read()
                f.close()
                m = re.search(
                    r"Scan of ([\w.]+):([\d]+) \[([/\w]+)\] \(([\w.]+)\)", data)
                self.hostname = m.group(1)
                self.port = m.group(2)
                self.uri = m.group(3)
                self.ipaddress = m.group(4)

                m = re.search("Server header:\n\n([\\w\\W]+)\n\n\n", data)
                self.header = m.group(1)

                self.items = []

                pattern = r'\((POST|GET)\): ([\w\W]*?) \]--'

                for m in re.finditer(pattern, data, re.DOTALL):

                    method = m.group(1)
                    info = re.search(
                        "^([\\w\\W]+)\\(([\\w\\W]+)\\)\n--\\[ ([\\w\\W]+)$", m.group(2))

                    vuln = {'method': m.group(1), 'desc': info.group(
                        1), 'url': info.group(2), 'resp': info.group(3)}
                    self.items.append(vuln)

            except SyntaxError as err:
                return None


class WebfuzzerPlugin(PluginBase):
    """
    Example plugin to parse webfuzzer output.
    """

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.id = "Webfuzzer"
        self.name = "Webfuzzer Output Plugin"
        self.plugin_version = "0.0.2"
        self.version = "0.2.0"
        self.options = None
        self._current_output = None
        self.host = None
        self._command_regex = re.compile(r'^(sudo webfuzzer|webfuzzer|\.\/webfuzzer)\s+.*?')
        self._completition = {'': '__Usage: ./webfuzzer -G|-P URL [OPTIONS]',
                              '-G': '<url>	get this as starting url (with parameters)',
                              '-P': '<url>	post this as starting url (with parameters)',
                              '-x': 'html output (txt default)',
                              '-c': 'use cookies',
                              '-C': '<cookies>	set this cookie(s) **',
                              '-s': 'check for sql, asp, vb, php errors (default)',
                              '-d': 'check for directory traversal *',
                              '-p': 'check for insecure perl open or xss *',
                              '-e': 'check for execution through shell escapes or xss *',
                              '-a': 'set all of the above switches on *',
                              }

        self._output_path = None

    def parseOutputString(self, output):
        """
        This method will discard the output the shell sends, it will read it from
        the xml where it expects it to be present.
        """

        if self._output_path is None:
            return False
        else:
            if not os.path.exists(self._output_path):
                return False

            parser = WebfuzzerParser(self._output_path)
            h_id = self.createAndAddHost(parser.ipaddress, hostnames=[parser.hostname])
            first = True
            for item in parser.items:
                if first:
                    s_id = self.createAndAddServiceToHost(h_id, parser.port, "tcp", ports=[parser.port])
                    first = False
                v_id = self.createAndAddVulnWebToService(h_id, s_id, name=item['desc'],path=item['url'],
                                                         response=item['resp'], method=item['method'],
                                                         website=parser.hostname)

        del parser

        return True

    def processCommandString(self, username, current_path, command_string):
        """
        """
        super().processCommandString(username, current_path, command_string)
        host = re.search(r"\-([G|P]) ([\w\.\-]+)", command_string)
        if host is not None:
            self.host = host.group(2)
            self._output_path = current_path + "/" + self.host + ".txt"


def createPlugin(*args, **kwargs):
    return WebfuzzerPlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2013  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information
"""
from faraday_plugins.plugins.plugin import PluginBase
import re
from collections import defaultdict

__author__ = "Francisco Amato"
__copyright__ = "Copyright (c) 2013, Infobyte LLC"
__credits__ = ["Francisco Amato"]
__license__ = ""
__version__ = "1.0.0"
__maintainer__ = "Francisco Amato"
__email__ = "famato@infobytesec.com"
__status__ = "Development"


class HydraParser:
    """
    The objective of this class is to parse an xml file generated by the hydra tool.

    @param hydra_filepath A proper simple report generated by hydra
    """

    def __init__(self, xml_output):
        lines = xml_output.splitlines()
        self.items = []
        for line in lines:

            reg = re.search(
                r"\[([^$]+)\]\[([^$]+)\] host: ([^$]+)   login: ([^$]+)   password: ([^$]+)",
                line)

            if reg:
                item = {
                    'port': reg.group(1),
                    'plugin': reg.group(2),
                    'ip': reg.group(3),
                    'login': reg.group(4),
                    'password': reg.group(5)}

                self.items.append(item)


class HydraPlugin(PluginBase):
    """
    Example plugin to parse hydra output.
    """

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.id = "Hydra"
        self.name = "Hydra XML Output Plugin"
        self.plugin_version = "0.0.1"
        self.version = "7.5"
        self.options = None
        self._command_regex = re.compile(r'^(sudo hydra|sudo \.\/hydra|hydra|\.\/hydra)\s+.*?')
        self.host = None
        self._use_temp_file = True
        self._temp_file_extension = "txt"
        self.xml_arg_re = re.compile(r"^.*(-o\s*[^\s]+).*$")

    def parseOutputString(self, output):
        """
        This method will discard the output the shell sends, it will read it from
        the xml where it expects it to be present.

        NOTE: if 'debug' is true then it is being run from a test case and the
        output being sent is valid.
        """

        parser = HydraParser(output)
        hosts = defaultdict(list)
        service = ''
        port = ''

        for item in parser.items:

            service = item['plugin']
            port = item['port']
            hosts[item['ip']].append([item['login'], item['password']])

        for k, v in hosts.items():
            ip = self.resolve_hostname(k)
            if ip != k:
                hostnames = [k]
            else:
                hostnames = None
            h_id = self.createAndAddHost(ip, hostnames=hostnames)
            s_id = self.createAndAddServiceToHost(
                h_id,
                service,
                ports=[port],
                protocol="tcp",
                status="open")

            for cred in v:
                self.createAndAddCredToService(
                    h_id,
                    s_id,
                    cred[0],
                    cred[1])

                self.createAndAddVulnToService(
                    h_id,
                    s_id,
                    "Weak Credentials",
                    f"[hydra found the following credentials]\nuser:{cred[0]}\npass:{cred[1]}",
                    severity="high")

        del parser

    def processCommandString(self, username, current_path, command_string):
        super().processCommandString(username, current_path, command_string)
        arg_match = self.xml_arg_re.match(command_string)
        if arg_match is None:
            return re.sub(r"(^.*?hydra?)", r"\1 -o %s" % self._output_file_path, command_string)
        else:
            return re.sub(arg_match.group(1), r"-o %s" % self._output_file_path, command_string)

    def _isIPV4(self, ip):
        if len(ip.split(".")) == 4:
            return True
        else:
            return False




def createPlugin(*args, **kwargs):
    return HydraPlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2013  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information

"""
import re
import xml.etree.ElementTree as ET
from urllib.parse import urlparse

from faraday_plugins.plugins.plugin import PluginXMLFormat
from faraday_plugins.plugins.plugins_utils import CWE_regex

__author__ = "Francisco Amato"
__copyright__ = "Copyright (c) 2013, Infobyte LLC"
__credits__ = ["Francisco Amato"]
__license__ = ""
__version__ = "1.0.0"
__maintainer__ = "Francisco Amato"
__email__ = "famato@infobytesec.com"
__status__ = "Development"


class WapitiXmlParser:
    """
    The objective of this class is to parse an xml file generated by the wapiti tool.

    TODO: Handle errors.
    TODO: Test wapiti output version. Handle what happens if the parser doesn't support it.
    TODO: Test cases.

    @param wapiti_xml_filepath A proper xml generated by wapiti
    """

    def __init__(self, xml_output, resolve_hostname):
        self.resolve_hostname = resolve_hostname
        tree = self.parse_xml(xml_output)
        if tree:
            self.items = [data for data in self.get_items(tree)]
        else:
            self.items = []

    def parse_xml(self, xml_output):
        """
        Open and parse an xml file.

        TODO: Write custom parser to just read the nodes that we need instead of
        reading the whole file.

        @return xml_tree An xml tree instance. None if error.
        """
        try:
            tree = ET.fromstring(xml_output)
        except SyntaxError as err:
            print(f"SyntaxError: {err}. {xml_output}")
            return None

        return tree

    def get_items(self, tree):
        """
        @return items A list of Host instances
        """

        yield Item(tree, self.resolve_hostname)


def get_attrib_from_subnode(xml_node, subnode_xpath_expr, attrib_name):
    """
    Finds a subnode in the item node and the retrieves a value from it

    @return An attribute value
    """

    node = xml_node.find(subnode_xpath_expr)
    if node is not None:
        return node.get(attrib_name)
    return None


class Item:
    """
    An abstract representation of a Item

    TODO: Consider evaluating the attributes lazily
    TODO: Write what's expected to be present in the nodes
    TODO: Refactor both Host and the Port clases?

    @param item_node A item_node taken from an wapiti xml tree
    """

    def __init__(self, item_node, resolve_hostname):
        self.node = item_node
        self.url = self.get_url(item_node)
        if self.url.hostname is not None:
            self.ip = resolve_hostname(self.url.hostname)
        else:
            self.ip = '0.0.0.0'
        self.hostname = self.url.hostname
        self.port = self.get_port(self.url)
        self.scheme = self.url.scheme
        self.vulns = self.get_vulns(item_node)

    def do_clean(self, value):
        myreturn = ""
        if value is not None:
            myreturn = re.sub("\n", "", value)
        return myreturn

    def get_text_from_subnode(self, node, subnode_xpath_expr):
        """
        Finds a subnode in the host node and the retrieves a value from it.

        @return An attribute value
        """
        sub_node = node.find(subnode_xpath_expr)
        if sub_node is not None:
            return sub_node.text.strip()

        return None

    def get_url(self, item_node):
        target = self.get_info(item_node, 'target')
        return urlparse(target)

    def get_info(self, item_node, name):
        path = item_node.findall('report_infos/info')

        for item in path:
            if item.attrib['name'] == name:
                return item.text

    def get_port(self, url):
        if url.port:
            return url.port
        else:
            if url.scheme == "http":
                return "80"
            elif url.scheme == "https":
                return "443"

    def get_vulns(self, item_node):
        vulns_node = item_node.findall('vulnerabilities/vulnerability')
        vulns_list = []

        for vuln in vulns_node:
            vulns_dict = {}
            vulns_dict['id'] = vuln.attrib['name']
            vulns_dict['description'] = self.get_text_from_subnode(vuln, 'description')
            vulns_dict['solution'] = self.get_text_from_subnode(vuln, 'solution')
            vulns_dict['references'] = self.get_references(vuln)
            vulns_dict['cwe'] = self.get_cwe(vuln)
            vulns_dict['entries'] = self.get_entries(vuln)
            vulns_list.append(vulns_dict)

        return vulns_list

    def get_cwe(self, node):
        refs = node.findall('references/reference')
        for ref in refs:
            if CWE_regex.search(self.get_text_from_subnode(ref, 'title')):
                return [CWE_regex.search(self.get_text_from_subnode(ref, 'title')).group()]
        return []

    def get_references(self, node):
        refs = node.findall('references/reference')
        references_list = []
        for ref in refs:
            references_list.append('Title: ' + self.get_text_from_subnode(ref, 'title'))
            references_list.append('URL: ' + self.get_text_from_subnode(ref, 'url'))

        return references_list

    def get_entries(self, node):
        entries = node.findall('entries/entry')
        entries_list = []
        for entry in entries:
            entries_dict = {}
            entries_dict['method'] = self.get_text_from_subnode(entry, 'method')
            entries_dict['path'] = self.get_text_from_subnode(entry, 'path')
            entries_dict['level'] = self.severity_format(entry)
            entries_dict['parameter'] = self.get_text_from_subnode(entry, 'parameter')
            entries_dict['http_request'] = self.get_text_from_subnode(entry, 'http_request')
            entries_dict['curl_command'] = self.get_text_from_subnode(entry, 'curl_command')
            entries_list.append(entries_dict)

        return entries_list

    def severity_format(self, node):
        """
        Convert Nexpose severity format into Faraday API severity format

        @return a severity
        """
        severity = self.get_text_from_subnode(node, 'level')

        if severity == '1':
            return 'high'
        elif severity == '2':
            return 'medium'
        elif severity == '3':
            return 'low'


class WapitiPlugin(PluginXMLFormat):
    """
    Example plugin to parse wapiti output.
    """

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        #self.identifier_tag = "report"
        self.identifier_tag_attributes = {'type'}
        self.id = "Wapiti"
        self.name = "Wapiti XML Output Plugin"
        self.plugin_version = "0.0.1"
        self.version = "2.2.1"
        self.options = None
        self.protocol = None
        self.host = None
        self.port = "80"
        self._use_temp_file = True
        self._temp_file_extension = "xml"
        self.xml_arg_re = re.compile(r"^.*(-oX\s*[^\s]+).*$")
        self._command_regex = re.compile(
            r'^(python wapiti|wapiti|sudo wapiti|sudo wapiti\.py|wapiti\.py|python wapiti\.py|\.\/wapiti\.py|wapiti |\.'
            r'\.\/wapiti|python wapiti|python \.\/wapiti)\s+.*?')
        self._completition = {
            "": "python wapiti.py http://server.com/base/url/ [options]",
            "-s": "&lt;url&gt; ",
            "--start": "&lt;url&gt; ",
            "-x": "&lt;url&gt; ",
            "--exclude": "&lt;url&gt; ",
            "-p": "&lt;url_proxy&gt; ",
            "--proxy": "&lt;url_proxy&gt; ",
            "-c": " -c &lt;cookie_file&gt; ",
            "--cookie": "&lt;cookie_file&gt; ",
            "-t": "&lt;timeout&gt; ",
            "--timeout": "&lt;timeout&gt; ",
            "-a": "&lt;login%password&gt; ",
            "--auth": "&lt;login%password&gt; ",
            "-r": "&lt;parameter_name&gt; ",
            "--remove": "&lt;parameter_name&gt; ",
            "-n": "&lt;limit&gt; ",
            "--nice": "&lt;limit&gt; ",
            "-m": "&lt;module_options&gt; Set the modules and HTTP methods to use for attacks. Example: -m \"-all,"
                  "xss:get,exec:post\"",
            "--module": "&lt;module_options&gt; Set the modules and HTTP methods to use for attacks. Example: -m \"-"
                        "all,xss:get,exec:post\"",
            "-u": "Use color to highlight vulnerables parameters in output",
            "--underline": "Use color to highlight vulnerables parameters in output",
            "-v": "&lt;level&gt; ",
            "--verbose": "&lt;level&gt; ",
            "-b": "&lt;scope&gt;",
            "--scope": "&lt;scope&gt;",
            "-f": "&lt;type_file&gt; ",
            "--reportType": "&lt;type_file&gt; ",
            "-o": "&lt;output_file&gt; ",
            "--output": "&lt;output_file&gt; ",
            "-i": "&lt;file&gt;",
            "--continue": "&lt;file&gt;",
            "-k": "&lt;file&gt;",
            "--attack": "&lt;file&gt;",
            "-h": "To print this usage message",
            "--help": "To print this usage message",
        }

    def report_belongs_to(self, **kwargs):
        if super().report_belongs_to(**kwargs):
            report_path = kwargs.get("report_path", "")
            with open(report_path) as f:
                output = f.read()
            return re.search("Wapiti", output) is not None
        return False

    def parseOutputString(self, output):
        """
        This method will discard the output the shell sends, it will read it from
        the xml where it expects it to be present.
        """

        parser = WapitiXmlParser(output, self.resolve_hostname)
        for item in parser.items:
            host_id = self.createAndAddHost(item.ip, hostnames=[item.hostname])
            if item.port is None:
                port = 0
            else:
                port = item.port
            service_id = self.createAndAddServiceToHost(host_id, item.scheme, protocol='tcp', ports=[port])
            for vuln in item.vulns:
                for entry in vuln['entries']:
                    vuln_id = self.createAndAddVulnWebToService(host_id,
                                                                service_id,
                                                                vuln['id'],
                                                                desc=vuln['description'],
                                                                ref=vuln['references'],
                                                                resolution=vuln['solution'],
                                                                severity=entry['level'],
                                                                website=entry['curl_command'],
                                                                path=entry['path'],
                                                                request=entry['http_request'],
                                                                method=entry['method'],
                                                                params=entry['parameter'],
                                                                cwe=vuln["cwe"])

    def processCommandString(self, username, current_path, command_string):
        """
        Adds the -oX parameter to get xml output to the command string that the
        user has set.
        """
        super().processCommandString(username, current_path, command_string)
        host = re.search(
            r"(http|https|ftp)\://([a-zA-Z0-9\.\-]+(\:[a-zA-Z0-9\.&amp;%\$\-]+)*@)*((25[0-5]|2[0-4][0-9]|[0-1]{1}[0-9]"
            r"{2}|[1-9]{1}[0-9]{1}|[1-9])\.(25[0-5]|2[0-4][0-9]|[0-1]{1}[0-9]{2}|[1-9]{1}[0-9]{1}|[1-9]|0)\.(25[0-5]|2"
            r"[0-4][0-9]|[0-1]{1}[0-9]{2}|[1-9]{1}[0-9]{1}|[1-9]|0)\.(25[0-5]|2[0-4][0-9]|[0-1]{1}[0-9]{2}|[1-9]{1}[0-9]"
            r"{1}|[0-9])|localhost|([a-zA-Z0-9\-]+\.)*[a-zA-Z0-9\-]+\.(com|edu|gov|int|mil|net|org|biz|arpa|info|name|"
            "pro|aero|coop|museum|[a-zA-Z]{2}))[\\:]*([0-9]+)*([/]*($|[a-zA-Z0-9\\.\\,\\?\'\\\\+&amp;%\\$#\\=~_\\-]+)).*?$",
            command_string)

        self.protocol = host.group(1)
        self.host = host.group(4)
        if host.group(11) is not None:
            self.port = host.group(11)
        if self.protocol == 'https':
            self.port = 443
        self.logger.debug("host = %s, port = %s", self.host, self.port)
        return f"{command_string} -o {self._output_file_path} -f xml \n"


def createPlugin(*args, **kwargs):
    return WapitiPlugin(*args, **kwargs)



"""
Faraday Penetration Test IDE
Copyright (C) 2013  Infobyte LLC (http://www.infobytesec.com/)
See the file 'doc/LICENSE' for the license information

"""

# Author: @EzequielTBH

from faraday_plugins.plugins.plugin import PluginBase
import json
import re

__author__ = "@EzequielTBH"
__copyright__ = "Copyright 2015, @EzequielTBH"
__credits__ = "@EzequielTBH"
__license__ = "GPL v3"
__version__ = "1.0.0"


class pasteAnalyzerPlugin(PluginBase):

    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, **kwargs)
        self.id = "pasteAnalyzer"
        self.name = "pasteAnalyzer JSON Output Plugin"
        self.plugin_version = "1.0.0"
        self.command_string = ""
        self._command_regex = re.compile(
            r'^(pasteAnalyzer|python pasteAnalyzer.py|\./pasteAnalyzer.py|sudo python pasteAnalyzer.py|sudo \./pasteAnalyzer.py)\s+.*?')

    def parseOutputString(self, output):
        # Generating file name with full path.
        indexStart = self.command_string.find("-j") + 3
        fileJson = self.command_string[
            indexStart:self.command_string.find(" ", indexStart)]
        fileJson = self._current_path + "/" + fileJson
        try:
            with open(fileJson) as fileJ:
                results = json.loads(fileJ.read())
        except Exception as e:
            return
        if results == []:
            return
        # Configuration initial.
        hostId = self.createAndAddHost("pasteAnalyzer")
        serviceId = self.createAndAddServiceToHost(
            hostId,
            "Web",
            "TcpHTTP",
            ['80']
        )

        # Loading results.
        for i in range(0, len(results), 2):

            data = results[i + 1]
            description = ""

            for element in data:

                # Is Category
                if type(element) == str: #TODO bte arrray decode
                    description += element + ": "

                # Is a list with results!
                else:
                    for element2 in element:
                        description += "\n" + element2
            self.createAndAddVulnWebToService(
                hostId,
                serviceId,
                results[i],
                description
            )

    def processCommandString(self, username, current_path, command_string):
        super().processCommandString(username, current_path, command_string)

        if command_string.find("-j") < 0:
            command_string += " -j JSON_OUTPUT "

        self.command_string = command_string

        return command_string


def createPlugin(*args, **kwargs):
    return pasteAnalyzerPlugin(*args, **kwargs)
